# -*- coding: utf-8 -*-
def backup_entra(results, path, output, token, azure_token, args, exclude):
    """
    Imports all the backup functions and runs them
    """

    params = {
        "token": token,
        "azure_token": azure_token,
        "path": path,
        "filetype": output,
        "exclude": ["assignments"],
    }

    if args.interactiveauth:
        from .backup.Entra.DeviceRegistration import DeviceRegistrationBackupModule

        results.append(DeviceRegistrationBackupModule(**params).main())

    else:
        print(
            "***Device Registration Policy is only available with interactive auth***"
        )

    if "entraApplications" not in exclude:
        from .backup.Entra.Applications import ApplicationsBackupModule

        results.append(ApplicationsBackupModule(**params).main())

    if "entraAuthenticationMethods" not in exclude:
        from .backup.Entra.AuthenticationMethods import (
            AuthenticationMethodsBackupModule,
        )

        results.append(AuthenticationMethodsBackupModule(**params).main())

    if "entraAuthorizationPolicy" not in exclude:
        from .backup.Entra.AuthorizationPolicy import AuthorizationPolicyBackupModule

        results.append(AuthorizationPolicyBackupModule(**params).main())

    if "entraAuthenticationFlowsPolicy" not in exclude:
        from .backup.Entra.AuthenticationFlows import AuthenticationFlowsBackupModule

        results.append(AuthenticationFlowsBackupModule(**params).main())

    if "entraDomains" not in exclude:
        from .backup.Entra.Domains import DomainsBackupModule

        results.append(DomainsBackupModule(**params).main())

    if "entraExternalIdentitiesPolicy" not in exclude:
        from .backup.Entra.ExternalIdentities import (
            ExternalIdentitiesPolicyBackupModule,
        )

        results.append(ExternalIdentitiesPolicyBackupModule(**params).main())

    if "entraB2BPolicy" not in exclude:
        from .backup.Entra.B2B import B2BPolicyBackupModule

        results.append(B2BPolicyBackupModule(**params).main())

    if "entraGroupSettings" not in exclude:
        from .backup.Entra.GroupSettings import GroupSettingsBackupModule

        results.append(GroupSettingsBackupModule(**params).main())

    if "entraSecurityDefaults" not in exclude:
        from .backup.Entra.SecurityDefaults import SecurityDefaultsBackupModule

        results.append(SecurityDefaultsBackupModule(**params).main())

    if "entraSSPR" not in exclude:
        from .backup.Entra.SSPR import SSPRBackupModule

        results.append(SSPRBackupModule(**params).main())

    if "entraRoamingSettings" not in exclude:
        from .backup.Entra.RoamingSettings import RoamingSettingsBackupModule

        results.append(RoamingSettingsBackupModule(**params).main())



# -*- coding: utf-8 -*-
from ...intunecdlib.BaseBackupModule import BaseBackupModule


class EnrollmentStatusPageBackupModule(BaseBackupModule):
    """A class used to backup Intune Enrollment Status Page

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        LOG_MESSAGE (str): The message to log when backing up the data
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/deviceEnrollmentConfigurations"
    LOG_MESSAGE = "Backing up Enrollment Status Page: "
    APP_ENDPOINT = "/beta/deviceAppManagement/mobileApps"

    def __init__(self, *args, **kwargs):
        """Initializes the EnrollmentStatusPageBackupModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Enrollment Profiles/Windows/ESP/"
        self.assignment_endpoint = "deviceManagement/deviceEnrollmentConfigurations/"
        self.assignment_extra_url = "/assignments"
        self.audit = False

    def main(self) -> dict[str, any]:
        """The main method to backup the Scope Tags

        Returns:
            dict[str, any]: The results of the backup
        """
        try:
            self.graph_data = self.make_graph_request(
                endpoint=self.endpoint + self.CONFIG_ENDPOINT
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting Enrollment Status Page data from {self.endpoint + self.CONFIG_ENDPOINT}: {e}",
            )
            return None

        # Only get the windows10EnrollmentCompletionPageConfiguration from the data
        self.graph_data = [
            item
            for item in self.graph_data["value"]
            if item["@odata.type"]
            == "#microsoft.graph.windows10EnrollmentCompletionPageConfiguration"
        ]

        for item in self.graph_data:
            # Get the app names from the app ids
            if "selectedMobileAppIds" in item:
                app_ids = item["selectedMobileAppIds"]
                app_names = []
                for app_id in app_ids:
                    app_data = self.make_graph_request(
                        endpoint=f"{self.endpoint + self.APP_ENDPOINT}/{app_id}"
                    )
                    if app_data:
                        app = {
                            "name": app_data["displayName"],
                            "type": app_data["@odata.type"],
                        }
                        app_names.append(app)
                if app_names:
                    item.pop("selectedMobileAppIds", None)
                    item["selectedMobileAppNames"] = app_names

        try:
            self.results = self.process_data(
                data=self.graph_data,
                filetype=self.filetype,
                path=self.path,
                name_key="displayName",
                log_message=self.LOG_MESSAGE,
            )
        except Exception as e:
            self.log(
                tag="error", msg=f"Error processing Enrollment Status Page data: {e}"
            )
            return None

        return self.results



# -*- coding: utf-8 -*-
import importlib
import os
import sys
from concurrent.futures import ThreadPoolExecutor, as_completed

from .decorators import time_command

# Add the repo root to sys.path if running locally
if "src" in os.path.abspath(os.path.dirname(__file__)):
    sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


def import_backup_module(module_path: str):
    """
    Dynamically imports a backup module, handling both installed and local Git repository cases.
    """
    try:
        # Try importing as an installed package
        return importlib.import_module(module_path, package="IntuneCD")
    except ModuleNotFoundError:
        try:
            # If that fails, assume we're running locally and try direct relative import
            return importlib.import_module(module_path)
        except ModuleNotFoundError as e:
            print(f"[ERROR] Could not import {module_path}: {e}")
            return None


@time_command()
def backup_intune(
    results,
    path,
    output,
    exclude,
    token,
    prefix,
    append_id,
    args,
    max_workers,
    platforms,
):
    """
    Imports all the backup functions dynamically and runs them in parallel.
    """

    from .intunecdlib.process_scope_tags import ProcessScopeTags

    process_scope_tags = ProcessScopeTags(token)

    if "ScopeTags" not in exclude:
        scope_tags = process_scope_tags.get_scope_tags()
    else:
        scope_tags = None

    params = {
        "token": token,
        "path": path,
        "filetype": output,
        "audit": args.audit,
        "append_id": append_id,
        "exclude": exclude,
        "scope_tags": scope_tags,
        "prefix": prefix,
        "ignore_oma_settings": args.ignore_omasettings,
        "platforms": platforms,
    }

    # List of backup modules to dynamically import and execute
    backup_modules = [
        (
            "AppConfigurations",
            ".backup.Intune.AppConfiguration",
            "AppConfigurationBackupModule",
        ),
        (
            "AppProtection",
            ".backup.Intune.AppProtection",
            "AppProtectionBackupModule",
        ),
        ("APNs", ".backup.Intune.APNs", "APNSBackupModule"),
        (
            "AppleEnrollmentProfile",
            ".backup.Intune.AppleEnrollmentProfiles",
            "AppleEnrollmentProfilesBackupModule",
        ),
        (
            "Applications",
            ".backup.Intune.Applications",
            "ApplicationsBackupModule",
        ),
        (
            "DeviceManagementSettings",
            ".backup.Intune.DeviceManagementSettings",
            "DeviceManagementSettingsBackupModule",
        ),
        (
            "DeviceCompliancePolicies",
            ".backup.Intune.DeviceCompliance",
            "DeviceComplianceBackupModule",
        ),
        (
            "Profiles",
            ".backup.Intune.DeviceConfigurations",
            "DeviceConfigurationBackupModule",
        ),
        ("Compliance", ".backup.Intune.Compliance", "ComplianceBackupModule"),
        (
            "CompliancePartner",
            ".backup.Intune.CompliancePartner",
            "CompliancePartnerBackupModule",
        ),
        (
            "ComplianceScripts",
            ".backup.Intune.ComplianceScripts",
            "ComplianceScriptsBackupModule",
        ),
        (
            "CustomAttributes",
            ".backup.Intune.CustomAttributes",
            "CustomAttributesBackupModule",
        ),
        (
            "ConditionalAccess",
            ".backup.Intune.ConditionalAccess",
            "ConditionalAccessBackupModule",
        ),
        (
            "DeviceCategories",
            ".backup.Intune.DeviceCategories",
            "DeviceCategoriesBackupModule",
        ),
        (
            "EnrollmentConfigurations",
            ".backup.Intune.EnrollmentConfigurations",
            "EnrollmentConfigurationsBackupModule",
        ),
        (
            "EnrollmentStatusPage",
            ".backup.Intune.EnrollmentStatusPage",
            "EnrollmentStatusPageBackupModule",
        ),
        ("Filters", ".backup.Intune.Filters", "FiltersBackupModule"),
        (
            "GPOConfigurations",
            ".backup.Intune.GroupPolicyConfigurations",
            "GroupPolicyConfigurationsBackupModule",
        ),
        (
            "ManagedGooglePlay",
            ".backup.Intune.ManagedGooglePlay",
            "ManagedGooglePlayBackupModule",
        ),
        (
            "Intents",
            ".backup.Intune.ManagementIntents",
            "ManagementIntentsBackupModule",
        ),
        (
            "ManagementPartner",
            ".backup.Intune.ManagementPartner",
            "ManagementPartnerBackupModule",
        ),
        (
            "NotificationTemplate",
            ".backup.Intune.NotificationTemplates",
            "NotificationTemplateBackupModule",
        ),
        (
            "PowershellScripts",
            ".backup.Intune.PowershellScripts",
            "PowershellScriptsBackupModule",
        ),
        (
            "ProactiveRemediation",
            ".backup.Intune.ProactiveRemediation",
            "ProactiveRemediationScriptBackupModule",
        ),
        (
            "RemoteAssistancePartner",
            ".backup.Intune.RemoteAssistancePartner",
            "RemoteAssistancePartnerBackupModule",
        ),
        (
            "ReusablePolicySettings",
            ".backup.Intune.ReusableSettings",
            "ReusableSettingsBackupModule",
        ),
        ("Roles", ".backup.Intune.Roles", "RolesBackupModule"),
        ("ScopeTags", ".backup.Intune.ScopeTags", "ScopeTagsBackupModule"),
        (
            "SettingsCatalog",
            ".backup.Intune.SettingsCatalog",
            "SettingsCatalogBackupModule",
        ),
        (
            "ShellScripts",
            ".backup.Intune.ShellScripts",
            "ShellScriptsBackupModule",
        ),
        (
            "VPP",
            ".backup.Intune.VolumePurchaseProgram",
            "VPPBackupModule",
        ),
        (
            "windowsDriverUpdates",
            ".backup.Intune.WindowsDriverUpdates",
            "WindowsDriverUpdatesBackupModule",
        ),
        (
            "WindowsEnrollmentProfile",
            ".backup.Intune.WindowsEnrollmentProfile",
            "WindowsEnrollmentProfilesBackupModule",
        ),
        (
            "windowsFeatureUpdates",
            ".backup.Intune.WindowsFeatureUpdates",
            "WindowsFeatureUpdatesBackupModule",
        ),
        (
            "windowsQualityUpdates",
            ".backup.Intune.WindowsQualityUpdates",
            "WindowsQualityUpdatesBackupModule",
        ),
    ]

    if args.activationlock:
        backup_modules.append(
            (
                "ActivationLock",
                ".backup.Intune.Activationlock",
                "ActivationLockBackupModule",
            )
        )

    if args.autopilot:
        backup_modules.append(
            (
                "Autopilot",
                ".backup.Intune.AutopilotDevices",
                "AutopilotDevicesBackupModule",
            )
        )

    # Use ThreadPoolExecutor to run multiple modules in parallel
    with ThreadPoolExecutor(max_workers=max_workers) as executor:
        future_to_module = {}

        for exclude_key, module_path, class_name in backup_modules:
            if exclude_key not in exclude:
                module = import_backup_module(module_path)
                if module:
                    backup_class = getattr(module, class_name)
                    future = executor.submit(backup_class(**params).main)
                    future_to_module[future] = exclude_key

        # Collect results as tasks complete
        for future in as_completed(future_to_module):
            module_name = future_to_module[future]
            try:
                result = future.result()
                if result:
                    results.append(result)
            except Exception as e:
                print(f"[ERROR] {module_name} failed with exception: {e}")



# -*- coding: utf-8 -*-
import time
import os

DEBUG = os.environ.get("DEBUG", False)


def time_command():
    def wrapper(func):
        def wrapped(*args, **kwargs):
            if not DEBUG:
                return func(*args, **kwargs)
            start = time.time()
            result = func(*args, **kwargs)
            end = time.time()
            print(f"[DEBUG] {func.__name__} ran in: {round(end - start)} seconds")
            return result

        return wrapped

    return wrapper



# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class ExternalIdentitiesPolicyUpdateModule(BaseUpdateModule):
    """A class used to update Entra External Collaboration Settings

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/policies/externalidentitiespolicy"

    def __init__(self, *args, **kwargs):
        """Initializes the ExternalIdentitiesPolicyUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Entra/External Collaboration Settings/"
        self.config_type = "External Collaboration Settings"
        self.handle_assignment = False
        self.get_match = False
        self.url_append_id = False

    def main(self) -> dict[str, any]:
        """The main method to update the Entra data"""
        if self.path_exists():
            try:
                entra_data = self.get_downstream_data(self.CONFIG_ENDPOINT)
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            for filename in os.listdir(self.path):
                if not filename.startswith("external_identities_policy."):
                    continue
                repo_data = self.load_repo_data(filename)
                if repo_data:
                    self.name = "External Identities Policy"
                    diff_data = self.create_diff_data(self.name, self.config_type)
                    update_data = {
                        "allowDeletedIdentitiesDataRemoval": repo_data[
                            "allowDeletedIdentitiesDataRemoval"
                        ],
                        "allowExternalIdentitiesToLeave": repo_data[
                            "allowExternalIdentitiesToLeave"
                        ],
                    }

                    try:
                        self.process_update(
                            downstream_data=entra_data,
                            repo_data=repo_data,
                            method="patch",
                            status_code=204,
                            config_endpoint=self.CONFIG_ENDPOINT,
                            update_data=update_data,
                        )
                    except Exception as e:
                        self.log(
                            tag="error",
                            msg=f"Error updating {self.config_type} {self.name}: {e}",
                        )

                    self.set_diff_data(diff_data)
                    self.diff_summary.append(diff_data)
                    self.reset_diffs_and_count()

        return self.diff_summary



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
This module tests the archive function.
"""

import os
import unittest
from unittest.mock import patch

from testfixtures import TempDirectory

from src.IntuneCD.intunecdlib.archive import move_to_archive


class TestMoveToArchive(unittest.TestCase):
    """Test class for move_to_archive."""

    def setUp(self) -> None:
        self.directory = TempDirectory()
        self.directory.create()
        self.d = self.directory.makedir("fake_path")
        self.archive_folder = self.directory.path + "/__archive__"

        self.directory.write(
            "fake_path/test.json",
            '{"test": "test"}',
            encoding="utf-8",
        )

        self.directory.write(
            "Management Intents/test_intent.json",
            '{"test": "test"}',
            encoding="utf-8",
        )

        self.directory.write(
            "test.json",
            '{"test": "test"}',
            encoding="utf-8",
        )

        self.createdFile = ["test.json"]

        self.datetime = patch("src.IntuneCD.intunecdlib.archive.datetime")
        self.datetime = self.datetime.start()
        self.datetime.return_value = "2020-01-01_00-00-00"

    def tearDown(self):
        self.directory.cleanup()
        self.datetime.stop()

    def test_folder_exists(self):
        """The archive folder should be created."""

        move_to_archive(self.directory.path, self.createdFile, "json")

        self.assertEqual(os.path.exists(self.archive_folder), True)

    def test_file_moved(self):
        """The files should be moved to the archive folder."""

        self.createdFile = []

        move_to_archive(self.directory.path, self.createdFile, "json")

        self.assertEqual(
            os.path.exists(self.archive_folder + "/" + self.datetime + "/test.json"),
            True,
        )
        self.assertEqual(
            os.path.exists(
                self.archive_folder + "/" + self.datetime + "/test_intent.json"
            ),
            True,
        )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from ...intunecdlib.BaseBackupModule import BaseBackupModule


class AppleEnrollmentProfilesBackupModule(BaseBackupModule):
    """A class used to backup Intune Apple Enrollment Profiles

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        LOG_MESSAGE (str): The message to log when backing up the data
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/depOnboardingSettings/"
    LOG_MESSAGE = "Backing up Apple Enrollment Profile: "

    def __init__(self, *args, **kwargs):
        """Initializes the AppleEnrollmentProfilesBackupModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Enrollment Profiles/Apple/"
        self.audit_filter = "componentName eq 'Enrollment'"
        # Apple Enrollment Profiles has no assignments, so exclude assignments from the run
        self.has_assignments = False

    def main(self) -> dict[str, any]:
        """The main method to backup the Apple Enrollment Profiles

        Returns:
            dict[str, any]: The results of the backup
        """
        try:
            self.graph_data = self.make_graph_request(
                endpoint=self.endpoint + self.CONFIG_ENDPOINT
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting Apple Enrollment Profile data from {self.endpoint + self.CONFIG_ENDPOINT}: {e}",
            )
            return None

        items_ids = [item["id"] for item in self.graph_data["value"]]

        # get the details of each profile using batch request
        batch_profile_data = self.batch_request(
            items_ids,
            "deviceManagement/depOnboardingSettings/",
            "/enrollmentProfiles",
        )

        # get a list of the value key from the batch_profile_data
        data = [
            value
            for profile in batch_profile_data
            for value in profile["value"]
            if value is not None
        ]

        try:
            self.results = self.process_data(
                data=data,
                filetype=self.filetype,
                path=self.path,
                name_key="displayName",
                log_message=self.LOG_MESSAGE,
                audit_compare_info={"type": "resourceId", "value_key": "id"},
            )
        except Exception as e:
            self.log(
                tag="error", msg=f"Error processing Apple Enrollment Profile data: {e}"
            )
            return None

        return self.results



# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class ComplianceUpdateModule(BaseUpdateModule):
    """A class used to update Intune compliance

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/compliancePolicies/"

    def __init__(self, *args, **kwargs):
        """Initializes the ComplianceUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Compliance Policies/Policies/"
        self.assignment_endpoint = "/deviceManagement/compliancePolicies/"
        self.assignment_extra_url = "/assign"
        self.exclude_paths = [
            "root['scheduledActionsForRule']",
            "root['settings']",
            "root['settings@odata.context']",
            "root['detectionScriptName']",
            "root['scheduledActionConfigurations']",
            "root['scheduledActionConfigurations'][0]['id']",
            "root['assignments']",
            "root['scheduledActionsForRule'][0]['scheduledActionConfigurations']",
        ]

    def _remove_compliance_keys(self, data: dict) -> dict[str, any]:
        """Removes additional keys from the compliance data

        Args:
            data (dict): The data to remove the keys from

        Returns:
            dict[str, any]: The data with the keys removed
        """
        if isinstance(data, dict):
            return {
                k: self._remove_compliance_keys(v)
                for k, v in data.items()
                if k
                not in [
                    "settingValueTemplateReference",
                    "settingInstanceTemplateReference",
                    "note",
                    "settingCount",
                    "creationSource",
                    "settings@odata.context",
                    "detectionScriptName",
                ]
            }
        if isinstance(data, list):
            return [self._remove_compliance_keys(v) for v in data]

        return data

    def _get_detection_script_id_path(self, data: dict, path: list = None) -> list:
        """Gets the path of the detection script id

        Args:
            data (dict): The data to get the path from
            path (list, optional): The path to the detection script id. Defaults to None.

        Returns:
            list: The path of the detection script id
        """
        if path is None:
            path = []
        if isinstance(data, dict):
            for k, v in data.items():
                if isinstance(v, dict):
                    new_path = self._get_detection_script_id_path(v, path + [k])
                    if new_path is not None:
                        return new_path
                elif isinstance(v, list):
                    for i, item in enumerate(v):
                        new_path = self._get_detection_script_id_path(
                            item, path + [k, i]
                        )
                        if new_path is not None:
                            return new_path
                elif v == "linux_customcompliance_discoveryscript":
                    return path
        return None

    def _set_value_from_path(
        self, repo_data: dict, value: str, path: list
    ) -> dict[str, any]:
        """Sets the value from the path

        Args:
            repo_data (dict): The data to set the value in
            value (str): The value to set
            path (list): The path to set the value in

        Returns:
            dict[str, any]: The data with the value set
        """
        item = repo_data
        for key in path[:-1]:
            if isinstance(item, list):
                item = item[int(key)]
            else:
                item = item[key]
        if isinstance(item, list):
            item[int(path[-1])] = value
        else:
            item[path[-1]] = value
        return repo_data

    def _set_detection_script_id(self, repo_data: dict) -> dict[str, any]:
        """Sets the detection script id

        Args:
            repo_data (dict): The data to set the detection script id in

        Returns:
            dict[str, any]: The data with the detection script id set
        """
        # get detection script id
        script_id = self.make_graph_request(
            self.endpoint + "/beta/deviceManagement/reusablePolicySettings/",
            {"$filter": f"displayName eq '{repo_data['detectionScriptName']}'"},
        )
        if script_id.get("value"):
            script_id_path = self._get_detection_script_id_path(repo_data)
            script_id_path = script_id_path + ["simpleSettingValue", "value"]
            repo_data = self._set_value_from_path(
                repo_data, script_id["value"][0]["id"], script_id_path
            )

            return repo_data

        return False

    def _get_scheduledActionsForRule(self, intune_data: dict) -> dict[str, any]:
        """Gets the scheduled actions for the rule

        Args:
            intune_data (dict): The Intune data to get the scheduled actions for

        Returns:
            dict[str, any]: The Intune data with the scheduled actions for the rule
        """
        new_intune_data = []
        for item in intune_data["value"]:
            actions = self.make_graph_request(
                endpoint=self.endpoint
                + self.CONFIG_ENDPOINT
                + item["id"]
                + "/scheduledActionsForRule",
                params={"$expand": "scheduledActionConfigurations"},
            )
            for action in actions["value"]:
                self.remove_keys(action)

            item["scheduledActionsForRule"] = actions["value"]
            new_intune_data.append(item)

        return new_intune_data

    def _detection_script_check(self, repo_data: dict) -> None:
        """Checks if the detection script exists

        Args:
            repo_data (dict): The data to check the detection script for
        """
        if "detectionScriptName" in repo_data:
            script_name = repo_data["detectionScriptName"]
            repo_data = self._set_detection_script_id(repo_data)
            if repo_data is False:
                self.log(
                    tag="warning",
                    msg=f"Detection script {script_name} not found, Compliance Policy {self.name} not updated",
                )

        return repo_data

    def _scheduledActionsForRule_diff_check(self, repo_data: dict) -> None:
        """Checks the scheduled actions for the rule for differences

        Args:
            repo_data (dict): The data to check the scheduled actions for the rule for
        """
        for intune_action, repo_action in zip(
            self.downstream_object.get("scheduledActionsForRule"),
            repo_data["scheduledActionsForRule"],
        ):
            action_diff = self.get_diffs(repo_action, intune_action, None)

            if action_diff:
                self.update_downstream_data(
                    config_endpoint=self.endpoint
                    + self.CONFIG_ENDPOINT
                    + self.downstream_id
                    + "/setScheduledActions",
                    method="post",
                    data={"scheduledActions": repo_data["scheduledActionsForRule"]},
                    status_code=200,
                )

                self.update_diff_data(action_diff)

    def _settings_diff_check(self, repo_data: dict) -> None:
        """Checks the settings for differences

        Args:
            repo_data (dict): The data to check the settings for
        """
        if repo_data.get("settings"):
            for intune_setting, repo_setting in zip(
                self.downstream_object.get("settings"),
                repo_data["settings"],
            ):
                setting_diff = self.get_diffs(repo_setting, intune_setting, None)

                if setting_diff:
                    self.update_downstream_data(
                        config_endpoint=self.endpoint
                        + self.CONFIG_ENDPOINT
                        + self.downstream_id,
                        method="put",
                        data=repo_data,
                        status_code=204,
                    )

                    self.update_diff_data(setting_diff)

    def _get_notification_template_id(self, rule: dict[str, any]) -> dict[str, any]:
        """Gets the notification template for a rule

        Args:
            rule (dict[str, any]): The rule to get the notification template for

        Returns:
            dict[str, any]: The notification template
        """
        for action in rule["scheduledActionConfigurations"]:
            if action.get("notificationTemplateName"):
                notification_template = self.make_graph_request(
                    self.endpoint
                    + "/beta/deviceManagement/notificationMessageTemplates/",
                    params={
                        "$filter": f"displayName eq '{action['notificationTemplateName']}'"
                    },
                )
                if notification_template["value"]:
                    action["notificationTemplateId"] = notification_template["value"][
                        0
                    ]["id"]
                else:
                    action[
                        "notificationTemplateId"
                    ] = "00000000-0000-0000-0000-000000000000"

                action.pop("notificationTemplateName")

        return rule

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            self.params = {"$expand": "settings"}
            try:
                intune_data = self.get_downstream_data(self.CONFIG_ENDPOINT)
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None
            self.downstream_assignments = self.batch_assignment(
                intune_data["value"],
                self.assignment_endpoint,
                "/assignments",
            )

            for filename in os.listdir(self.path):
                # reset params
                self.params = None
                self.create_request = None
                self.notify = True
                self.config_type = "Compliance Policy"

                repo_data = self.load_repo_data(filename)
                if repo_data:
                    self.create_request = None
                    if "technologies" not in repo_data:
                        continue
                    self.match_info = {
                        "name": repo_data.get("name"),
                        "technologies": repo_data.get("technologies"),
                    }
                    self.name = repo_data.get("name")
                    diff_data = self.create_diff_data(self.name, self.config_type)

                    # Check if detection script exists
                    repo_data = self._detection_script_check(repo_data)
                    if repo_data is False:
                        continue
                    # Get the scheduled actions for the rule
                    intune_data["value"] = self._get_scheduledActionsForRule(
                        intune_data
                    )

                    for rule in repo_data.get("scheduledActionsForRule"):
                        self._get_notification_template_id(rule)

                    # Remove additional keys from the data
                    new_intune_data = []
                    for item in intune_data["value"]:
                        item = self._remove_compliance_keys(item)
                        new_intune_data.append(item)
                    intune_data["value"] = new_intune_data
                    repo_data = self._remove_compliance_keys(repo_data)

                    for item in intune_data["value"]:
                        for action in item["scheduledActionsForRule"][0][
                            "scheduledActionConfigurations"
                        ]:
                            self.remove_keys(action)

                    try:
                        self.process_update(
                            downstream_data=intune_data["value"],
                            repo_data=repo_data,
                            method="patch",
                            status_code=204,
                            config_endpoint=self.CONFIG_ENDPOINT,
                        )
                    except Exception as e:
                        self.log(
                            tag="error",
                            msg=f"Error updating {self.config_type} {self.name}: {e}",
                        )
                    # If a match is found, check for differences on the scheduled actions and settings
                    if self.downstream_object:
                        self.notify = False
                        self.config_type = "Compliance Policy Scheduled Actions"
                        self._scheduledActionsForRule_diff_check(repo_data)
                        self.config_type = "Compliance Policy Settings"
                        repo_data.pop("assignments", None)
                        self._settings_diff_check(repo_data)
                    # If a new compliance policy is created, set the scheduled actions
                    if self.create_request:
                        self.update_downstream_data(
                            config_endpoint=self.endpoint
                            + self.CONFIG_ENDPOINT
                            + self.create_request["id"]
                            + "/setScheduledActions",
                            method="post",
                            data={
                                "scheduledActions": repo_data["scheduledActionsForRule"]
                            },
                            status_code=200,
                        )

                    self.set_diff_data(diff_data)
                    self.diff_summary.append(diff_data)
                    self.reset_diffs_and_count()

            self.remove_downstream_data(self.CONFIG_ENDPOINT, intune_data["value"])

        return self.diff_summary



# -*- coding: utf-8 -*-
import json
import os

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class GroupPolicyConfigurationsUpdateModule(BaseUpdateModule):
    """A class used to update Intune GroupPolicyConfigurations data

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/groupPolicyConfigurations/"

    def __init__(self, *args, **kwargs):
        """Initializes the GroupPolicyConfigurationsUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Group Policy Configurations/"
        self.assignment_endpoint = "/deviceManagement/groupPolicyConfigurations/"
        self.assignment_extra_url = "/assign"
        self.exclude_paths = ["root['assignments']", "root['definitionValues']"]
        self.remove_status_code = 204

    class definition_values_json:
        """Class for creating json for definition values."""

        DEFINITION_ODATA_BIND = "https://graph.microsoft.com/beta/deviceManagement/groupPolicyDefinitions('{id}')"
        PRESENTATION_ODATA_BIND = "https://graph.microsoft.com/beta/deviceManagement/groupPolicyDefinitions('{id}')/presentations('{pid}')"

        def __init__(self, definition: dict, presentation: dict):
            self.definition = definition
            self.presentation = presentation
            self.request_json = self.request_json = {
                "added": [],
                "updated": [],
                "deletedIds": [],
            }

        def modify_definition(self, scenario, intune_definition_id):
            """
            Create the json for the definition values.

            Args:
                scenario (str): The scenario to which the definition value is added.
                intune_definition_id (str): The id of the definition value.
            Returns:
                dict: The json for the definition value.
            """
            d_id = self.definition["definition"]["id"]

            presentation_values = []

            for presentation in self.definition["presentationValues"]:
                if presentation["presentation"].get("required", False) is True:
                    default_presentation = {
                        "@odata.type": presentation["@odata.type"],
                        "value": presentation["value"],
                        "presentation@odata.bind": self.PRESENTATION_ODATA_BIND.replace(
                            "{id}", d_id
                        ).replace("{pid}", presentation["presentation"]["id"]),
                    }

                    presentation_values.append(default_presentation)

            self.request_json[f"{scenario}"].append(
                {
                    "enabled": self.definition["enabled"],
                    "definition@odata.bind": self.DEFINITION_ODATA_BIND.replace(
                        "{id}", d_id
                    ),
                    "presentationValues": presentation_values,
                }
            )

            if intune_definition_id:
                self.request_json[f"{scenario}"][0]["id"] = intune_definition_id

            return self.request_json

        def modify_presentation(self, scenario, defval_id, pid):
            """
            Create the json for the presentation values.

            Args:
                scenario (str): The scenario to which the presentation value is added.
                defval_id (str): The id of the definition value.
                pid (str): The id of the presentation value.

            Returns:
                dict: The json for the presentation value.
            """
            d_id = self.definition["definition"]["id"]
            pvid = self.presentation["presentation"]["id"]

            self.request_json[f"{scenario}"].append(
                {
                    "id": defval_id,
                    "enabled": self.definition["enabled"],
                    "definition@odata.bind": self.DEFINITION_ODATA_BIND.replace(
                        "{id}", d_id
                    ),
                    "presentationValues": [
                        {
                            "@odata.type": self.presentation["@odata.type"],
                            "presentation@odata.bind": self.PRESENTATION_ODATA_BIND.replace(
                                "{id}", d_id
                            ).replace(
                                "{pid}", pvid
                            ),
                        }
                    ],
                }
            )

            if pid:
                self.request_json[f"{scenario}"][0]["presentationValues"][0]["id"] = pid
            if self.presentation.get("values"):
                self.request_json[f"{scenario}"][0]["presentationValues"][0][
                    "values"
                ] = self.presentation["values"]
            if self.presentation.get("value"):
                self.request_json[f"{scenario}"][0]["presentationValues"][0][
                    "value"
                ] = self.presentation["value"]

            return self.request_json

    def custom_ingestion_match(self, data: dict) -> dict:
        """Match definitions from custom ingestion to definitions in Intune."""

        match = 0
        categories = self.make_graph_request(
            self.endpoint
            + "/beta/deviceManagement/groupPolicyCategories?$expand=definitions($select=id, displayName, categoryPath, classType)&$select=id, displayName&$filter=ingestionSource eq 'custom'",
        )
        # Go through each definition in repo data and compare to Intune data
        for definition in data["definitionValues"]:
            definition["definition"].pop("groupPolicyCategoryId", None)
            # Create string to compare
            repo_def_str = f'{definition["definition"]["classType"]}:{definition["definition"]["displayName"]}:{definition["definition"]["categoryPath"]}'
            # Go through each category and definition in Intune data
            for intune_definition in categories["value"]:
                for intune_definition in intune_definition["definitions"]:
                    # Create string to compare
                    intune_definition_str = f'{intune_definition["classType"]}:{intune_definition["displayName"]}:{intune_definition["categoryPath"]}'
                    # If the strings match, add the Intune definition id to the repo data
                    if repo_def_str == intune_definition_str:
                        definition["definition"]["id"] = intune_definition["id"]
                        # If the definition id was found, add 1 to match
                        match += 1

        if match == len(data["definitionValues"]):
            return data

        return False

    def find_matching_presentations(self, repo_data: dict, data: dict) -> dict:
        """Search for matching presentation values in repo data and Intune data."""
        for repo_def in repo_data.get("definitionValues", []):
            for intune_definition in data.get("definitionValues", []):
                for repo_pres in repo_def.get("presentationValues", []):
                    for intune_presentation in intune_definition.get(
                        "presentationValues", []
                    ):
                        repo_label = repo_pres.get("presentation", {}).get("label")
                        intune_label = intune_presentation.get("presentation", {}).get(
                            "label"
                        )
                        if repo_label == intune_label:
                            repo_pres["id"] = intune_presentation["id"]
                            repo_pres["presentation"]["id"] = intune_presentation[
                                "presentation"
                            ]["id"]

        return repo_data

    def post_presentation_values(
        self,
        definition: dict,
        presentation: dict,
        intune_id: str,
        scenario: str,
        pid: str,
    ):
        """
        Post presentation values to a group policy configuration

        Args:
            definition (dict): Definition values from repo
            presentation (dict): Presentation values from repo
            intune_id (str): Group policy configuration ID in Intune
            scenario (str): Scenario to use for API request
            pid (str): Presentation ID in Intune
        """
        defval_id = None

        # Get definition values from API
        def_vals = self.make_graph_request(
            self.endpoint
            + self.CONFIG_ENDPOINT
            + f"{intune_id}/definitionValues?$expand=definition"
        )

        # Find the matching definition value
        for def_val in def_vals["value"]:
            if def_val["definition"]["id"] == definition["definition"]["id"]:
                defval_id = def_val["id"]
                break

        # If no matching definition value is found, print a message and return
        if defval_id is None:
            label = presentation.get("presentation", {}).get("label", "None")
            self.log(
                tag="warning",
                msg=f"No matching definition found for presentation value '{label}'. Skipping...",
            )
            return

        # Prepare definition values data for API request
        json_data = self.definition_values_json(
            definition=definition, presentation=presentation
        )
        json_data.modify_presentation(scenario=scenario, defval_id=defval_id, pid=pid)
        request_data = json.dumps(json_data.request_json)

        # Make API request to update definition values
        self.make_graph_request(
            endpoint=self.endpoint
            + self.CONFIG_ENDPOINT
            + f"{intune_id}/updateDefinitionValues/",
            method="post",
            data=request_data,
            status_code=204,
        )

    def post_definition_values(
        self, definition: dict, intune_id: str, scenario: str, intune_definition_id: str
    ):
        """
        Post definition values to a group policy configuration

        Args:
            definition (dict): Definition values from repo
            intune_id (str): Group policy configuration ID in Intune
            scenario (str): Scenario to use for API request
            intune_definition_id (str): Definition ID in Intune
        """
        # Prepare definition values data for API request
        json_data = self.definition_values_json(
            definition=definition, presentation=None
        )
        json_data.modify_definition(
            scenario=scenario, intune_definition_id=intune_definition_id
        )
        request_data = json.dumps(json_data.request_json)

        # Make API request to update definition values
        self.make_graph_request(
            endpoint=self.endpoint
            + self.CONFIG_ENDPOINT
            + f"{intune_id}/updateDefinitionValues/",
            method="post",
            data=request_data,
            status_code=204,
        )

    def update_definition(
        self, repo_data: dict, data: dict, intune_id: str, intune_definition_ids: list
    ):
        """
        Update definition and presentation values in a group policy configuration

        Args:
            repo_data (dict): Group policy configuration data from repo
            data (dict): Group policy configuration data from Intune
            intune_id (str): Group policy configuration ID in Intune
            intune_definition_ids (str): Definition IDs in Intune
        """
        self.notify = False
        # Go through each definition value in repo data
        for definition in repo_data.get("definitionValues", []):
            def_id = definition["definition"]["id"]
            # If definition does not exist in data or not in intune_definition_ids, add it
            if not data.get("definitionValues") or def_id not in intune_definition_ids:
                self.log(msg="Adding definition values")
                self.post_definition_values(definition, intune_id, "added", None)
                for presentation in definition.get("presentationValues", []):
                    if presentation["presentation"].get("required", False) is False:
                        self.log(msg="Adding presentation values")
                        self.post_presentation_values(
                            definition, presentation, intune_id, "updated", None
                        )

            # Go through each definition value in data
            for intune_definition in data.get("definitionValues", []):
                intune_definition_id = intune_definition["id"]
                # If definition exists in data, compare and update if necessary
                if (
                    definition["definition"]["id"]
                    == intune_definition["definition"]["id"]
                ):
                    definition = self.remove_keys(definition)
                    # self.config_type = "Definition Value"
                    # self.name = definition["definition"]["displayName"]
                    definition_diff = self.get_diffs(
                        definition,
                        intune_definition,
                        [
                            "root['presentationValues']",
                            "root['definition']['groupPolicyCategoryId']",
                            "root['lastModifiedDateTime']",
                        ],
                    )

                    # If there are differences, update the definition
                    if definition_diff:
                        self.message = "Updating definition values, values changed:"
                        self.update_diff_data(definition_diff)
                        self.post_definition_values(
                            definition, intune_id, "updated", intune_definition_id
                        )

                    for presentation in definition.get("presentationValues", []):
                        for intune_presentation in intune_definition.get(
                            "presentationValues", []
                        ):
                            presentation_id = intune_presentation["id"]
                            if (
                                presentation["presentation"]["id"]
                                == intune_presentation["presentation"]["id"]
                            ):
                                self.get_pop_keys(
                                    presentation,
                                    ["id", "lastModifiedDateTime", "createdDateTime"],
                                    "pop",
                                )
                                self.get_pop_keys(
                                    presentation["presentation"],
                                    ["lastModifiedDateTime", "createdDateTime"],
                                    "pop",
                                )

                                presentation_diff = self.get_diffs(
                                    presentation,
                                    intune_presentation,
                                    ["root['lastModifiedDateTime']"],
                                )

                                # If there are differences, update the presentation value
                                if presentation_diff:
                                    self.message = (
                                        "Updating presentation values, values changed:"
                                    )
                                    self.update_diff_data(presentation_diff)
                                    self.post_presentation_values(
                                        definition,
                                        presentation,
                                        intune_id,
                                        "updated",
                                        presentation_id,
                                    )

                            elif (
                                definition["definition"]["id"]
                                == intune_definition["definition"]["id"]
                                and intune_definition["presentationValues"] is None
                            ):
                                self.log(msg="Adding presentation values")
                                self.post_presentation_values(
                                    definition,
                                    presentation,
                                    intune_id,
                                    "added",
                                    None,
                                )

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            try:
                intune_data = self.get_downstream_data(self.CONFIG_ENDPOINT)
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            self.downstream_assignments = self.batch_assignment(
                intune_data["value"],
                self.assignment_endpoint,
                "/assignments",
            )

            intune_profiles = []
            for profile in intune_data["value"]:
                definitions = self.make_graph_request(
                    endpoint=self.endpoint
                    + self.CONFIG_ENDPOINT
                    + profile["id"]
                    + "/definitionValues",
                    params={"$expand": "definition"},
                )

                if not definitions:
                    continue

                profile["definitionValues"] = definitions["value"]

                for definition in profile["definitionValues"]:
                    presentation = self.make_graph_request(
                        endpoint=self.endpoint
                        + self.CONFIG_ENDPOINT
                        + profile["id"]
                        + "/definitionValues/"
                        + definition["id"]
                        + "/presentationValues",
                        params={"$expand": "presentation"},
                    )
                    definition["presentationValues"] = presentation["value"]

                intune_profiles.append(profile)

            for filename in os.listdir(self.path):
                repo_data = self.load_repo_data(filename)
                if repo_data:
                    self.create_request = None
                    self.config_type = "Group Policy Configuration"
                    self.notify = True
                    self.match_info = {
                        "displayName": repo_data.get("displayName"),
                        "policyConfigurationIngestionType": repo_data.get(
                            "policyConfigurationIngestionType"
                        ),
                    }
                    self.name = repo_data.get("displayName")
                    diff_data = self.create_diff_data(self.name, self.config_type)
                    update_data = {
                        "displayName": repo_data["displayName"],
                        "description": repo_data["description"],
                        "roleScopeTagIds": repo_data["roleScopeTagIds"],
                    }

                    if repo_data["policyConfigurationIngestionType"] == "custom":
                        repo_data = self.custom_ingestion_match(repo_data)
                        if not repo_data:
                            self.log(
                                tag="warning",
                                msg=f"Some definitions was not found for {self.name}, import custom ADMX files to Intune first.",
                            )
                            continue

                    try:
                        self.process_update(
                            downstream_data=intune_profiles,
                            repo_data=repo_data,
                            method="patch",
                            status_code=200,
                            config_endpoint=self.CONFIG_ENDPOINT,
                            update_data=update_data,
                        )
                    except Exception as e:
                        self.log(
                            tag="error",
                            msg=f"Error updating {self.config_type} {self.name}: {e}",
                        )

                    if self.downstream_object:
                        intune_definition_ids = [
                            val
                            for val in self.downstream_object["definitionValues"]
                            for key, val in val["definition"].items()
                            if key == "id"
                        ]

                        if repo_data["policyConfigurationIngestionType"] == "custom":
                            repo_data = self.custom_ingestion_match(repo_data)
                            repo_data = self.find_matching_presentations(
                                repo_data, self.downstream_object
                            )

                        self.update_definition(
                            repo_data,
                            self.downstream_object,
                            self.downstream_id,
                            intune_definition_ids,
                        )

                    if self.create_request:
                        if repo_data["policyConfigurationIngestionType"] == "custom":
                            for definition in repo_data.get("definitionValues", []):
                                definition["presentationValues"] = []

                        for definition in repo_data["definitionValues"]:
                            self.post_definition_values(
                                definition,
                                self.create_request["id"],
                                "added",
                                None,
                            )

                            for presentation in definition["presentationValues"]:
                                if (
                                    presentation["presentation"].get("required", False)
                                    is False
                                ):
                                    self.post_presentation_values(
                                        definition,
                                        presentation,
                                        self.create_request["id"],
                                        "updated",
                                        None,
                                    )

                    self.set_diff_data(diff_data)
                    self.diff_summary.append(diff_data)
                    self.reset_diffs_and_count()

            self.remove_downstream_data(self.CONFIG_ENDPOINT, intune_profiles)

        return self.diff_summary



# -*- coding: utf-8 -*-
from .intunecdlib.documentation_functions import document_configs
from .decorators import time_command


@time_command()
def document_entra(configpath, outpath, maxlength, split, cleanup, decode):
    """
    This function is used to document Entra settings.
    """
    document_configs(
        f"{configpath}/Entra/Applications",
        outpath,
        "Applications",
        maxlength,
        split,
        cleanup,
        decode,
    )

    document_configs(
        f"{configpath}/Entra/Authentication Methods",
        outpath,
        "Authentication Methods",
        maxlength,
        split,
        cleanup,
        decode,
    )

    document_configs(
        f"{configpath}/Entra/Authentication Flows Policy",
        outpath,
        "Authentication Flows Policy",
        maxlength,
        split,
        cleanup,
        decode,
    )

    document_configs(
        f"{configpath}/Entra/Authorization Policy",
        outpath,
        "Authorization Policy",
        maxlength,
        split,
        cleanup,
        decode,
    )

    document_configs(
        f"{configpath}/Entra/Device Registration Policy",
        outpath,
        "Device Registration Policy",
        maxlength,
        split,
        cleanup,
        decode,
    )

    document_configs(
        f"{configpath}/Entra/Domains",
        outpath,
        "Domains",
        maxlength,
        split,
        cleanup,
        decode,
    )

    document_configs(
        f"{configpath}/Entra/External Collaboration Settings",
        outpath,
        "External Collaboration Settings",
        maxlength,
        split,
        cleanup,
        decode,
    )

    document_configs(
        f"{configpath}/Entra/Group Settings",
        outpath,
        "Group Settings",
        maxlength,
        split,
        cleanup,
        decode,
    )

    document_configs(
        f"{configpath}/Entra/Password Reset Policies",
        outpath,
        "Password Reset Policies",
        maxlength,
        split,
        cleanup,
        decode,
    )

    document_configs(
        f"{configpath}/Entra/Roaming Settings",
        outpath,
        "Roaming Settings",
        maxlength,
        split,
        cleanup,
        decode,
    )

    document_configs(
        f"{configpath}/Entra/Security Defaults",
        outpath,
        "Security Defaults",
        maxlength,
        split,
        cleanup,
        decode,
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
This module contains all functions for the documentation.
"""

import base64
import binascii
import glob
import json
import os
import platform
import re

import yaml
from pytablewriter import MarkdownTableWriter


def md_file(outpath):
    """
    This function creates the markdown file.

    :param outpath: The path to save the Markdown document to
    """
    if not os.path.exists(f"{outpath}"):
        open(outpath, "w+", encoding="utf-8").close()
    else:
        open(outpath, "w", encoding="utf-8").close()


def write_table(data):
    """
    This function creates the markdown table.

    :param data: The data to be written to the table
    :return: The Markdown table writer
    """

    writer = MarkdownTableWriter(
        headers=["setting", "value"],
        value_matrix=data,
    )

    return writer


def escape_markdown(text):
    """
    This function escapes markdown characters.

    :param text: The text to be escaped
    :return: The escaped text
    """

    # Escape markdown characters
    parse = re.sub(r"([\_*\[\]()\{\}`>\#\+\-=|\.!])", r"\\\1", text)

    return parse


def assignment_table(data):
    """
    This function creates the Markdown assignments table.

    :param data: The data to be written to the table
    :return: The Markdown table writer
    """

    def write_assignment_table(data, headers):
        writer = MarkdownTableWriter(headers=headers, value_matrix=data)

        return writer

    table = ""
    if "assignments" in data:
        assignments = data["assignments"]
        assignment_list = []
        target = ""
        intent = ""
        for assignment in assignments:
            if (
                assignment["target"]["@odata.type"]
                == "#microsoft.graph.allDevicesAssignmentTarget"
            ):
                target = "All Devices"
            if (
                assignment["target"]["@odata.type"]
                == "#microsoft.graph.allLicensedUsersAssignmentTarget"
            ):
                target = "All Users"
            if "groupName" in assignment["target"]:
                target = assignment["target"]["groupName"]
            if "intent" in assignment:
                intent = assignment["intent"]
                headers = ["intent", "target", "filter type", "filter name"]
            else:
                headers = ["target", "filter type", "filter name"]
            if intent:
                assignment_list.append(
                    [
                        intent,
                        target,
                        assignment["target"][
                            "deviceAndAppManagementAssignmentFilterType"
                        ],
                        assignment["target"][
                            "deviceAndAppManagementAssignmentFilterId"
                        ],
                    ]
                )
            else:
                assignment_list.append(
                    [
                        target,
                        assignment["target"][
                            "deviceAndAppManagementAssignmentFilterType"
                        ],
                        assignment["target"][
                            "deviceAndAppManagementAssignmentFilterId"
                        ],
                    ]
                )

            table = write_assignment_table(assignment_list, headers)

    return table


def remove_characters(string):
    """
    This function removes characters from the string.
    :param string: The string to be cleaned
    :return: The cleaned string
    """

    remove_chars = '#@}{]["'
    for char in remove_chars:
        string = string.replace(char, "")

    return string


def is_base64(s):
    """Check if a string is a valid base64-encoded string"""
    try:
        # Attempt to decode the string
        if isinstance(s, str):
            decoded = base64.b64decode(s.encode())
        else:
            decoded = base64.b64decode(s)
        # If decoding succeeds and the decoded bytes match the original string, it's a valid base64-encoded string
        return decoded == s.encode()
    except (TypeError, binascii.Error):
        # If decoding fails, it's not a valid base64-encoded string
        return False


def decode_base64(data):
    """
    This function decodes the data if it is base64 encoded.
    :param data: The data to be decoded
    :return: The decoded data
    """

    try:
        return base64.b64decode(data).decode("utf-8")
    except (base64.binascii.Error, UnicodeDecodeError):
        raise ValueError("Unable to decode data")


def clean_list(data, decode):
    """
    This function returns a list with strings to be used in a table.
    :param data: The data to be cleaned
    :return: The list of strings
    """

    def list_to_string(item_list) -> str:
        string = ""
        for i in item_list:
            if isinstance(i, (str, int, bool)):
                if decode and is_base64(i):
                    i = decode_base64(i)
                string += f"<li> {i} </li>"
            elif isinstance(i, dict):
                string += dict_to_string(i)
            else:
                string += i

        return string

    def dict_to_string(d) -> str:
        string = ""
        for key, val in d.items():
            if isinstance(val, list):
                string += f"**{key}:** <ul>"
                string += list_to_string(val)
                string += "</ul>"
            elif isinstance(val, dict):
                string += dict_to_ul(val)
            else:
                string += simple_value_to_string(key, val)

        string += "<br/>"

        return string

    def dict_to_ul(val) -> str:
        string = ""
        for k, v in val.items():
            if isinstance(v, list):
                string += f"**{k}:** <ul>"
                string += list_to_string(v)
                string += "</ul>"
            elif isinstance(v, dict):
                string += f"**{k}:** <ul>"
                string += dict_to_ul(v)
                string += "</ul>"
            else:
                string += simple_value_to_string(k, v)
        return string

    def simple_value_to_string(key, val) -> str:
        if decode and is_base64(val):
            val = decode_base64(val)

        if isinstance(val, str):
            val = val.replace("\\", "\\\\")

        return f"**{key}:** {val}<br/>"

    def list_string(item_list) -> str:
        string = ""
        for i in item_list:
            if isinstance(i, (str, int, bool)):
                if decode and is_base64(i):
                    i = decode_base64(i)
                string += f"{i}<br/>"
            if isinstance(i, list):
                string += list_to_string(i)
            if isinstance(i, dict):
                string += dict_to_string(i)

        return string

    def string(s) -> str:
        if decode and is_base64(s):
            s = decode_base64(s)

        if len(s) > 200:
            string = f"<details><summary>Click to expand...</summary>{s}</details>"
        else:
            string = s

        return string

    values = []

    for item in data:
        if isinstance(item, list):
            values.append(list_string(item))
        elif isinstance(item, dict):
            values.append(dict_to_ul(item))
        elif isinstance(item, str):
            values.append(string(item))
        elif isinstance(item, (bool, int)):
            values.append(item)
        else:
            values.append(item)

    return values


def write_type_header(split, outpath, header):
    """
    This function writes the header to the Markdown document.

    :param outpath: The path to save the Markdown document to
    :param header: Header of the configuration being documented
    """
    if not split:
        with open(outpath, "a", encoding="utf-8") as md:
            md.write("# " + header + "\n")


def document_configs(
    configpath,
    outpath,
    header,
    max_length,
    split,
    cleanup,
    decode,
    split_per_config=False,
):
    """
    Documents configurations, optionally splitting by type or per config.

    :param configpath: Path to backup files
    :param outpath: Base path for Markdown output
    :param header: Configuration type header (e.g., "AppConfigurations")
    :param max_length: Max length for displayed values
    :param split: Split into one file per type
    :param cleanup: Remove empty values
    :param decode: Decode base64 values
    :param split_per_config: Split into one file per individual config
    """
    if not os.path.exists(configpath):
        return

    # Base file for non-split or type-split mode
    if split and not split_per_config:
        outpath = os.path.join(configpath, f"{header}.md")
        md_file(outpath)

    if split_per_config is False:
        with open(outpath, "a", encoding="utf-8") as md:
            md.write("## " + header + "\n")

    # Use recursive pattern to catch deeper structures
    pattern = os.path.join(
        configpath, "**", "*.[jy][sa][mo][nl]"
    )  # Matches .json, .yaml, .yml
    files = sorted(glob.glob(pattern, recursive=True), key=str.casefold)
    if not files:
        return

    for filename in files:
        if (
            filename.endswith(".md")
            or os.path.isdir(filename)
            or os.path.basename(filename) == ".DS_Store"
        ):
            continue

        try:
            # Load data
            with open(filename, encoding="utf-8") as f:
                if filename.endswith((".yaml", ".yml")):
                    repo_data = json.loads(json.dumps(yaml.safe_load(f)))
                elif filename.endswith(".json"):
                    repo_data = json.load(f)
                else:
                    continue

            # Prepare assignments table
            assignments_table = assignment_table(repo_data)
            repo_data.pop("assignments", None)

            # Handle description
            description = repo_data.pop("description", "") or ""

            # Build config table
            config_table_list = []
            for key, value in zip(
                repo_data.keys(), clean_list(repo_data.values(), decode)
            ):
                if cleanup and not value and not isinstance(value, bool):
                    continue
                if key == "@odata.type":
                    key = "Odata type"
                else:
                    key = " ".join(re.findall("[A-Z][^A-Z]*", key[0].upper() + key[1:]))
                if max_length and isinstance(value, str) and len(value) > max_length:
                    value = "Value too long to display"
                config_table_list.append([key, value])

            config_table = write_table(config_table_list)

            # Determine output file and header
            config_name = repo_data.get(
                "displayName",
                repo_data.get(
                    "name",
                    os.path.splitext(os.path.basename(filename))[0]
                    .replace("_", " ")
                    .title(),
                ),
            )
            if split_per_config:
                # One file per config
                safe_config_name = re.sub(
                    r'[<>:"/\\|?*]', "_", config_name
                )  # Sanitize filename
                if not os.path.exists(f"{configpath}/docs"):
                    os.makedirs(f"{configpath}/docs")
                config_outpath = os.path.join(
                    f"{configpath}/docs", f"{safe_config_name}.md"
                )
                md_file(config_outpath)
                target_md = config_outpath
                top_header = f"# {config_name}"
                split_per_config_index_md(configpath, header)
            elif split:
                # One file per type
                target_md = outpath
                top_header = f"### {config_name}"
            else:
                # Single file
                target_md = outpath
                top_header = f"### {config_name}"

            # Write to file
            with open(target_md, "a", encoding="utf-8") as md:
                md.write(top_header + "\n")
                if description:
                    md.write(f"Description: {escape_markdown(description)}\n")
                if assignments_table:
                    md.write("#### Assignments\n")
                    md.write(str(assignments_table) + "\n")
                md.write("#### Configuration\n")
                md.write(str(config_table) + "\n")

        except Exception as e:
            print(f"[DEBUG] Error processing {filename}: {type(e).__name__}: {e}")


def document_management_intents(configpath, outpath, header, split):
    """
    This function documents the management intents.

    :param configpath: The path to where the backup files are saved
    :param outpath: The path to save the Markdown document to
    :param header: Header of the configuration being documented
    :param split: Split documentation into multiple files
    """

    # If configurations path exists, continue
    if os.path.exists(configpath):
        if split:
            outpath = configpath + "/" + header + ".md"
            md_file(outpath)

        with open(outpath, "a", encoding="utf-8") as md:
            md.write("## " + header + "\n")

        pattern = configpath + "*/*"
        for filename in sorted(glob.glob(pattern, recursive=True), key=str.casefold):
            # If path is Directory, skip
            if os.path.isdir(filename):
                continue
            # If file is .DS_Store, skip
            if filename == ".DS_Store":
                continue

            # Check which format the file is saved as then open file, load data and set query parameter
            with open(filename, encoding="utf-8") as f:
                if filename.endswith(".yaml"):
                    data = json.dumps(yaml.safe_load(f))
                    repo_data = json.loads(data)
                elif filename.endswith(".json"):
                    f = open(filename, encoding="utf-8")
                    repo_data = json.load(f)

                # Create assignments table
                assignments_table = ""
                assignments_table = assignment_table(repo_data)
                repo_data.pop("assignments", None)

                intent_settings_list = []
                for setting in repo_data["settingsDelta"]:
                    setting_definition = setting["definitionId"].split("_")[1]
                    setting_definition = (
                        setting_definition[0].upper() + setting_definition[1:]
                    )
                    setting_definition = re.findall("[A-Z][^A-Z]*", setting_definition)
                    setting_definition = " ".join(setting_definition)

                    vals = []
                    value = str(remove_characters(setting["valueJson"]))
                    comma = re.findall("[:][^:]*", value)
                    for v in value.split(","):
                        v = v.replace(" ", "")
                        if comma:
                            v = f'**{v.replace(":", ":** ")}'
                        vals.append(v)
                    value = ",".join(vals)
                    value = value.replace(",", "<br />")

                    intent_settings_list.append([setting_definition, value])

                repo_data.pop("settingsDelta")

                description = ""
                if "description" in repo_data:
                    if repo_data["description"] is not None:
                        description = repo_data["description"]
                        repo_data.pop("description")

                intent_table_list = []

                for key, value in zip(
                    repo_data.keys(), clean_list(repo_data.values(), decode=False)
                ):
                    key = key[0].upper() + key[1:]
                    key = re.findall("[A-Z][^A-Z]*", key)
                    key = " ".join(key)

                    if value and isinstance(value, str):
                        if len(value.split(",")) > 1:
                            vals = []
                            for v in value.split(","):
                                v = v.replace(" ", "")
                                v = f'**{v.replace(":", ":** ")}'
                                vals.append(v)
                            value = ",".join(vals)
                            value = value.replace(",", "<br />")

                    intent_table_list.append([key, value])

                table = intent_table_list + intent_settings_list

                config_table = write_table(table)
                # Write data to file
                with open(outpath, "a", encoding="utf-8") as md:
                    if "displayName" in repo_data:
                        md.write("### " + repo_data["displayName"] + "\n")
                    if "name" in repo_data:
                        md.write("### " + repo_data["name"] + "\n")
                    if description:
                        md.write(f"Description: {escape_markdown(description)} \n")
                    if assignments_table:
                        md.write("#### Assignments \n")
                        md.write(str(assignments_table) + "\n")
                    md.write("#### Configuration \n")
                    md.write(str(config_table) + "\n")


def split_per_config_index_md(configpath, header):
    """
    This function creates an index Markdown file for split_per_config mode.
    :param configpath: The path to where the backup files are saved
    :param outpath: The path to save the Markdown document to
    :param header: Header of the configuration being documented
    """
    # get all md files from the docs directory
    files = get_docs_md_files(configpath)
    index_md = f"{configpath}/{header}.md"
    md_file(index_md)

    with open(index_md, "w", encoding="utf-8") as doc:
        l1 = f"# {header} \n\n"
        l2 = "## File index \n\n"
        doc.writelines([l1, l2])
        for file in files:
            doc.writelines(
                [
                    "[",
                    str(file).split("/")[-1],
                    "](",
                    str(file).replace(" ", "%20"),
                    ") \n\n",
                ]
            )


def get_docs_md_files(configpath):
    """
    This function gets the Markdown files in the configpath/docs directory.
    :return: List of Markdown files
    """
    slash = "/"
    md_files = []
    client_os = platform.uname().system
    mdpath = configpath + "/docs/*.md"
    if client_os == "Windows":
        slash = "\\"
    for filename in glob.glob(mdpath):
        md_files.append(f"./docs/{filename.split(slash)[-1]}")
    return md_files


def get_md_files(configpath):
    """
    This function gets the Markdown files in the configpath directory.
    :return: List of Markdown files
    """
    slash = "/"
    client_os = platform.uname().system
    if client_os == "Windows":
        slash = "\\"
    md_files = []
    patterns = ["*/*.md", "*/*/*.md", "*/*/*/*.md", "*/*/*/*/*.md"]
    for pattern in patterns:
        for filename in glob.glob(configpath + pattern, recursive=True):
            # if folder name is docs, skip
            if "docs" in filename:
                continue
            filepath = filename.split(slash)
            configpathname = configpath.split(slash)[-1]
            filepath = filepath[filepath.index(configpathname) :]
            filepath = "/".join(filepath[1:])
            ignore_files = ["README", "index", "prod-as-built"]
            file_basename = os.path.splitext(filepath.rsplit("/", maxsplit=1)[-1])[0]
            if file_basename not in ignore_files:
                md_files.append(f"./{filepath}")
    # Sort the list alphabetically by file name without extension, case-insensitive
    md_files.sort(key=lambda f: os.path.splitext(os.path.basename(f))[0].lower())

    return md_files



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
This module is used to get the access token for the tenant.
"""

import json
import os

from .get_accesstoken import (
    obtain_accesstoken_app,
    obtain_accesstoken_cert,
    obtain_accesstoken_interactive,
)


def getAuth(mode, localauth, certauth, interactiveauth, scopes, entra, tenant):
    """
    This function authenticates to MS Graph and returns the access token.

    :param mode: The mode used when using this tool
    :param localauth: Path to dict with keys to authenticate
    :param tenant: Which tenant to authenticate to, PROD or DEV
    :return: The access token
    """

    if certauth:
        KEY_FILE = os.environ.get("KEY_FILE")
        THUMBPRINT = os.environ.get("THUMBPRINT")
        TENANT_NAME = os.environ.get("TENANT_NAME")
        CLIENT_ID = os.environ.get("CLIENT_ID")
        PASSPHRASE = os.environ.get("PASSPHRASE")

        if not all([KEY_FILE, THUMBPRINT, TENANT_NAME, CLIENT_ID]):
            raise ValueError("One or more os.environ variables not set")
        return obtain_accesstoken_cert(
            TENANT_NAME, CLIENT_ID, THUMBPRINT, KEY_FILE, PASSPHRASE
        )

    if interactiveauth:
        TENANT_NAME = os.environ.get("TENANT_NAME")
        CLIENT_ID = os.environ.get("CLIENT_ID")

        if not all([TENANT_NAME, CLIENT_ID]):
            raise ValueError("One or more os.environ variables not set")

        return obtain_accesstoken_interactive(TENANT_NAME, CLIENT_ID, scopes)

    if mode:
        if mode == "devtoprod":
            if localauth:
                with open(localauth, encoding="utf-8") as json_data:
                    auth_dict = json.load(json_data)
                tenant_TENANT_NAME = auth_dict["params"][tenant + "_TENANT_NAME"]
                tenant_CLIENT_ID = auth_dict["params"][tenant + "_CLIENT_ID"]
                tenant_CLIENT_SECRET = auth_dict["params"][tenant + "_CLIENT_SECRET"]
                if entra:
                    os.environ["TENANT_ID"] = auth_dict["params"][tenant + "_TENANT_ID"]
                    os.environ["KEY"] = auth_dict["params"].get("KEY")
            else:
                tenant_TENANT_NAME = os.environ.get(tenant + "_TENANT_NAME")
                tenant_CLIENT_ID = os.environ.get(tenant + "_CLIENT_ID")
                tenant_CLIENT_SECRET = os.environ.get(tenant + "_CLIENT_SECRET")
                if entra:
                    os.environ["TENANT_ID"] = os.environ.get(tenant + "_TENANT_ID")
            if not all([tenant_TENANT_NAME, tenant_CLIENT_ID, tenant_CLIENT_SECRET]):
                raise ValueError(
                    "One or more os.environ variables for " + tenant + " not set"
                )

            return obtain_accesstoken_app(
                tenant_TENANT_NAME, tenant_CLIENT_ID, tenant_CLIENT_SECRET
            )

        if mode == "standalone":
            if localauth:
                with open(localauth, encoding="utf-8") as json_data:
                    auth_dict = json.load(json_data)
                TENANT_NAME = auth_dict["params"]["TENANT_NAME"]
                CLIENT_ID = auth_dict["params"]["CLIENT_ID"]
                CLIENT_SECRET = auth_dict["params"]["CLIENT_SECRET"]
                if entra:
                    os.environ["TENANT_ID"] = auth_dict["params"]["TENANT_ID"]
                    os.environ["KEY"] = auth_dict["params"].get("KEY")
            else:
                TENANT_NAME = os.environ.get("TENANT_NAME")
                CLIENT_ID = os.environ.get("CLIENT_ID")
                CLIENT_SECRET = os.environ.get("CLIENT_SECRET")
                if entra:
                    os.environ["TENANT_ID"] = os.environ.get("TENANT_ID")
            if not all([TENANT_NAME, CLIENT_ID, CLIENT_SECRET]):
                raise ValueError("One or more os.environ variables not set")

            return obtain_accesstoken_app(TENANT_NAME, CLIENT_ID, CLIENT_SECRET)



# -*- coding: utf-8 -*-

import argparse
from IntuneCD.run_backup import get_parser as get_backup_parser, start as run_backup
from IntuneCD.run_documentation import (
    get_parser as get_documentation_parser,
    start as run_documentation,
)
from IntuneCD.run_update import get_parser as get_update_parser, start as run_update
from importlib.metadata import version, PackageNotFoundError


def get_version():
    try:
        return version("IntuneCD")
    except PackageNotFoundError:
        return "unknown"


def banner():
    green = "\033[92m"
    bold = "\033[1m"
    reset = "\033[0m"
    return (
        bold
        + green
        + r"""
          ..
        ....
       .::::
      .:::::            ___       _                     ____ ____
     .::::::           |_ _|_ __ | |_ _   _ _ __   ___ / ___|  _ \
    .:::::::.           | || '_ \| __| | | | '_ \ / _ \ |   | | | |
   ::::::::::::::.      | || | | | |_| |_| | | | |  __/ |___| |_| |
  ::::::::::::::.      |___|_| |_|\__|\__,_|_| |_|\___|\____|____/                 _
        :::::::.       |_ _|_ __ | |_ _   _ _ __   ___    __ _ ___    ___ ___   __| | ___
        ::::::.         | || '_ \| __| | | | '_ \ / _ \  / _` / __|  / __/ _ \ / _` |/ _ \
        :::::.          | || | | | |_| |_| | | | |  __/ | (_| \__ \ | (_| (_) | (_| |  __/
        ::::           |___|_| |_|\__|\__,_|_| |_|\___|  \__,_|___/  \___\___/ \__,_|\___|
        :::
        ::

Keep your Intune setup version-controlled and auditable. 
IntuneCD brings Intune to your CI/CD pipeline and command line with automated backups, updates, and documentation.
    """
        + reset
    )


class BannerHelpFormatter(argparse.RawTextHelpFormatter):
    def add_usage(self, usage, actions, groups, prefix=None):
        if prefix is None:
            prefix = ""
        banner_text = banner()
        super().add_usage(
            banner_text + "\n\n" + (prefix or "Usage: "), actions, groups, ""
        )


def main():
    parser = argparse.ArgumentParser(formatter_class=BannerHelpFormatter)
    subparsers = parser.add_subparsers(dest="command", required=True)

    parser.add_argument(
        "-v", "--version", action="version", version=f"IntuneCD {get_version()}"
    )

    backup_parser = subparsers.add_parser(
        "backup", parents=[get_backup_parser(include_help=False)]
    )
    backup_parser.set_defaults(func=run_backup)

    update_parser = subparsers.add_parser(
        "update", parents=[get_update_parser(include_help=False)]
    )
    update_parser.set_defaults(func=run_update)

    documentation_parser = subparsers.add_parser(
        "document", parents=[get_documentation_parser(include_help=False)]
    )
    documentation_parser.set_defaults(func=run_documentation)

    args = parser.parse_args()
    args.func(args)



# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class AuthenticationMethodsConfigurationsUpdateModule(BaseUpdateModule):
    """A class used to update Entrata Authentication Methods

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/policies/authenticationMethodsPolicy"

    def __init__(self, *args, **kwargs):
        """Initializes the TemplateUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Entra/Authentication Methods/"
        self.config_type = "Authentication Methods Configuration"
        self.handle_assignment = False
        self.exclude_paths = [
            "root['id']",
            "root['featureSettings']['numberMatchingRequiredState']",
        ]
        self.get_match = False

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            try:
                entra_data = self.get_downstream_data(self.CONFIG_ENDPOINT)
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            for filename in os.listdir(self.path):
                repo_data = self.load_repo_data(filename)
                if repo_data:
                    for entra_config, repo_config in zip(
                        entra_data["authenticationMethodConfigurations"],
                        repo_data["authenticationMethodConfigurations"],
                    ):
                        self.name = repo_config["id"]
                        diff_data = self.create_diff_data(self.name, self.config_type)
                        config_endpoint = f"{self.CONFIG_ENDPOINT}/authenticationMethodConfigurations/"
                        self.get_pop_keys(
                            entra_config,
                            [
                                "featureSettings.numberMatchingRequiredState",
                            ],
                        )

                        try:
                            self.process_update(
                                downstream_data=entra_config,
                                repo_data=repo_config,
                                method="patch",
                                status_code=204,
                                config_endpoint=config_endpoint,
                            )
                        except Exception as e:
                            self.log(
                                tag="error",
                                msg=f"Error updating {self.config_type} {self.name}: {e}",
                            )

                        self.set_diff_data(diff_data)
                        self.diff_summary.append(diff_data)
                        self.reset_diffs_and_count()

        return self.diff_summary



# -*- coding: utf-8 -*-
from concurrent.futures import ThreadPoolExecutor, as_completed

from .intunecdlib.documentation_functions import (
    document_configs,
    document_management_intents,
)
from .decorators import time_command


@time_command()
def document_intune(
    configpath,
    outpath,
    maxlength,
    split,
    cleanup,
    decode,
    split_per_config,
    max_workers,
):
    """
    This function is used to document Intune configuration using threading.

    :param configpath: Path where backup files are stored
    :param outpath: Path to save the Markdown documentation
    :param maxlength: Maximum length of values in the documentation
    :param split: Determines if documentation should be split into multiple files
    :param cleanup: Remove empty values from documentation
    :param decode: Decode base64 values
    :param split_per_config: Whether to split each config into its own Markdown file
    :param max_workers: Maximum number of concurrent threads
    """

    # Ensure the output directory exists
    # os.makedirs(outpath, exist_ok=True)

    # List of documentation tasks
    doc_tasks = [
        ("App Configuration", "App Configuration"),
        ("App Protection", "App Protection"),
        ("Apple Push Notification", "Apple Push Notification"),
        ("Apple VPP Tokens", "Apple VPP Tokens"),
        ("Applications/iOS", "iOS Applications"),
        ("Applications/macOS", "macOS Applications"),
        ("Applications/Android", "Android Applications"),
        ("Applications/Windows", "Windows Applications"),
        ("Applications/Web App", "Web Applications"),
        ("Applications/Office Suite", "Office Suite Applications"),
        ("Compliance Policies/Policies", "Compliance Policies"),
        ("Compliance Policies/Scripts", "Compliance Scripts"),
        ("Compliance Policies/Message Templates", "Message Templates"),
        ("Conditional Access", "Conditional Access"),
        ("Device Configurations", "Configuration Profiles"),
        ("Device Management Settings", "Device Management Settings"),
        ("Group Policy Configurations", "Group Policy Configurations"),
        ("Enrollment Profiles/Apple", "Apple Enrollment Profiles"),
        ("Enrollment Profiles/Windows", "Windows Enrollment Profiles"),
        ("Enrollment Profiles/Windows/ESP", "Enrollment Status Page"),
        ("Enrollment Configurations", "Enrollment Configurations"),
        ("Device Categories", "Device Categories"),
        ("Filters", "Filters"),
        ("Managed Google Play", "Managed Google Play"),
        ("Partner Connections", "Partner Connections"),
        ("Proactive Remediations", "Proactive Remediations"),
        ("Scripts/Shell", "Shell Scripts"),
        ("Custom Attributes", "Custom Attributes"),
        ("Scripts/Powershell", "Powershell Scripts"),
        ("Settings Catalog", "Settings Catalog"),
        ("Driver Updates", "Windows Driver Updates"),
        ("Feature Updates", "Windows Feature Updates"),
        ("Quality Updates", "Windows Quality Updates"),
        ("Roles", "Roles"),
        ("Scope Tags", "Scope Tags"),
    ]

    # sort doc_tasks alphabetically
    doc_tasks = sorted(doc_tasks, key=lambda x: x[1])

    if split or split_per_config:
        with ThreadPoolExecutor(max_workers=max_workers) as executor:
            futures = {
                executor.submit(
                    document_configs,
                    f"{configpath}/{task[0]}",
                    outpath,
                    task[1],
                    maxlength,
                    split,
                    cleanup,
                    decode,
                    split_per_config,
                ): task[1]
                for task in doc_tasks
            }

            for future in as_completed(futures):
                task_name = futures[future]
                try:
                    future.result()
                except Exception as e:
                    print(f"Error processing {task_name}: {e}")

    else:
        # Run sequentially if split options are disabled
        for task in doc_tasks:
            document_configs(
                f"{configpath}/{task[0]}",
                outpath,
                task[1],
                maxlength,
                split,
                cleanup,
                decode,
                split_per_config,
            )

    # **Run Management Intents Sequentially**
    document_management_intents(
        f"{configpath}/Management Intents/", outpath, "Management Intents", split
    )



# -*- coding: utf-8 -*-
from .decorators import time_command


@time_command()
def update_entra(diff_summary, path, token, azure_token, report, args, exclude):
    """
    Imports all the update functions and runs them
    """

    params = {
        "path": path,
        "token": token,
        "report": report,
        "exclude": exclude,
        "azure_token": azure_token,
    }
    if args.interactiveauth:
        from .update.Entra.DeviceRegistration import (
            DeviceRegistrationPolicyUpdateModule,
        )

        diff_summary.append(DeviceRegistrationPolicyUpdateModule(**params).main())
    else:
        print(
            "***Device Registration Policy is only available with interactive auth***"
        )

    if "entraAuthenticationMethodsPolicy" not in exclude:
        from .update.Entra.AuthenticationMethodsPolicy import (
            AuthenticationMethodsPolicyUpdateModule,
        )

        diff_summary.append(AuthenticationMethodsPolicyUpdateModule(**params).main())

    if "entraAuthenticationMethodsConfigurations" not in exclude:
        from .update.Entra.AuthenticationMethodsConfigurations import (
            AuthenticationMethodsConfigurationsUpdateModule,
        )

        diff_summary.append(
            AuthenticationMethodsConfigurationsUpdateModule(**params).main()
        )

    if "entraAuthorizationPolicy" not in exclude:
        from .update.Entra.AuthorizationPolicy import AuthorizationPolicyUpdateModule

        diff_summary.append(AuthorizationPolicyUpdateModule(**params).main())

    if "entraAuthenticationFlowsPolicy" not in exclude:
        from .update.Entra.AuthenticationFlows import (
            AuthenticationFlowsPolicyUpdateModule,
        )

        diff_summary.append(AuthenticationFlowsPolicyUpdateModule(**params).main())

    if "entraExternalIdentitiesPolicy" not in exclude:
        from .update.Entra.ExternalIdentitiesPolicy import (
            ExternalIdentitiesPolicyUpdateModule,
        )

        diff_summary.append(ExternalIdentitiesPolicyUpdateModule(**params).main())

    if "entraB2BPolicy" not in exclude:
        from .update.Entra.B2B import B2BUpdateModule

        diff_summary.append(B2BUpdateModule(**params).main())

    if "entraDomains" not in exclude:
        from .update.Entra.Domains import DomainsUpdateModule

        diff_summary.append(DomainsUpdateModule(**params).main())

    if "entraGroupSettings" not in exclude:
        from .update.Entra.GroupSettings import GroupSettingsUpdateModule

        diff_summary.append(GroupSettingsUpdateModule(**params).main())

    if "entraSecurityDefaults" not in exclude:
        from .update.Entra.SecurityDefaults import SecurityDefaultsUpdateModule

        diff_summary.append(SecurityDefaultsUpdateModule(**params).main())

    if "entraRoamingSettings" not in exclude:
        from .update.Entra.RoamingSettings import RoamingSettingsUpdateModule

        diff_summary.append(RoamingSettingsUpdateModule(**params).main())

    if "entraSSPR" not in exclude:
        from .update.Entra.SSPR import SSPRUpdateModule

        diff_summary.append(SSPRUpdateModule(**params).main())



# -*- coding: utf-8 -*-
from ...intunecdlib.BaseBackupModule import BaseBackupModule


class ApplicationsBackupModule(BaseBackupModule):
    """A class used to backup Intune Applications

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        LOG_MESSAGE (str): The message to log when backing up the data
    """

    CONFIG_ENDPOINT = "/beta/deviceAppManagement/mobileApps"
    LOG_MESSAGE = "Backing up Application: "

    def __init__(self, *args, **kwargs):
        """Initializes the ApplicationsBackupModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Applications/"
        self.audit_filter = self.audit_filter or "componentName eq 'MobileApp'"
        self.assignment_endpoint = (
            self.assignment_endpoint or "deviceAppManagement/mobileApps/"
        )
        self.assignment_extra_url = self.assignment_extra_url or "/assignments"
        self.config_audit_data = True

    def main(self) -> dict[str, any]:
        """The main method to backup the Applications

        Returns:
            dict[str, any]: The results of the backup
        """
        try:
            self.graph_data = self.make_graph_request(
                endpoint=self.endpoint + self.CONFIG_ENDPOINT,
                params={
                    "$filter": "(microsoft.graph.managedApp/appAvailability) eq null or (microsoft.graph.managedApp/appAvailability) "
                    "eq 'lineOfBusiness' or isAssigned eq true"
                },
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting Application data from {self.endpoint + self.CONFIG_ENDPOINT}: {e}",
            )
            return None

        app_ids = [app["id"] for app in self.graph_data["value"]]
        scope_tag_responses = self.batch_request(
            app_ids, "deviceAppManagement/mobileApps/", "?$select=roleScopeTagIds,id"
        )

        # create a list of dicts with the app id and the response
        self.app_ids = [{"id": app_id} for app_id in app_ids]
        # as we must process each app individually, get the assignment data up front
        self.assignment_responses = self.batch_assignment(
            self.app_ids, self.assignment_endpoint, self.assignment_extra_url
        )

        # as we must process each app individually, get the audit data up front
        if self.audit:
            self.audit_data = self.make_audit_request(self.audit_filter)

        for app in self.graph_data["value"]:
            platform = None
            base_path = self.path
            app.pop("description", None)
            scope_tag_data = [v for v in scope_tag_responses if app["id"] == v["id"]]
            if scope_tag_data:
                app["roleScopeTagIds"] = scope_tag_data[0]["roleScopeTagIds"]

            def generate_app_name(app, app_type, suffix=""):
                try:
                    return app["displayName"] + "_" + app_type + suffix
                except Exception as e:
                    self.log(
                        tag="error",
                        msg=f"Error generating app name for {app['id']}: {e}",
                    )
                    return app["displayName"]

            platform_mapping = {
                "ios": "iOS",
                "macos": "macOS",
                "android": "Android",
                "windows": "Windows",
                "microsoft": "Windows",
                "win32": "Windows",
                "office": "Office Suite",
                "webApp": "Web App",
            }

            app_type = app["@odata.type"]
            app_type_lower = app_type.lower()

            for key, value in platform_mapping.items():
                if key in app_type_lower:
                    platform = value
                    break

            if app_type == "#microsoft.graph.iosVppApp":
                app_name = generate_app_name(
                    app, "iOSVppApp", "_" + str(app["vppTokenAppleId"].split("@")[0])
                )
            elif app_type == "#microsoft.graph.macOsVppApp":
                app_name = generate_app_name(
                    app, "macOSVppApp", "_" + str(app["vppTokenAppleId"].split("@")[0])
                )
            elif app_type == "#microsoft.graph.win32LobApp":
                suffix = (
                    "_Win32"
                    if not app["displayVersion"]
                    else "_" + str(app["displayVersion"]).replace(".", "_")
                )
                app_name = generate_app_name(app, "Win32", suffix)
            elif app_type == "#microsoft.graph.windowsMobileMSI":
                app_name = generate_app_name(
                    app, "WinMSI", "_" + str(app["productVersion"]).replace(".", "_")
                )
            else:
                app_name = generate_app_name(app, app_type.split(".")[2])

            self.preset_filename = app_name

            self.path = f"{self.path}{platform}/"

            try:
                app_results = self.process_data(
                    data=app,
                    filetype=self.filetype,
                    path=self.path,
                    name_key="displayName",
                    log_message=self.LOG_MESSAGE,
                    audit_compare_info={"type": "resourceId", "value_key": "id"},
                )
                self.update_results(app_results)
            except Exception as e:
                self.log(tag="error", msg=f"Error processing Application data: {e}")
                return None

            self.path = base_path

        return self.results



# -*- coding: utf-8 -*-
from ...intunecdlib.BaseBackupModule import BaseBackupModule


class ActivationLockBackupModule(BaseBackupModule):
    """A class used to backup Intune Activation Lock Bypass Codes

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        LOG_MESSAGE (str): The message to log when backing up the data
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/managedDevices"
    LOG_MESSAGE = "Backing up "

    def __init__(self, *args, **kwargs):
        """Initializes the ActivationLockBackupModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Activation Lock Bypass Codes/"
        self.append_id = False
        self.audit = False
        self.prefix = None
        self.preset_filename = "activation_lock_bypass_codes"
        self.has_assignments = False

    def main(self) -> None:
        """The main method to backup the Activation Lock Bypass Codes"""
        self.log(msg="Backing up Activation Lock Bypass Codes")

        try:
            self.graph_data = self.make_graph_request(
                endpoint=self.endpoint + self.CONFIG_ENDPOINT,
                params={
                    "$select": "id",
                    "$filter": "startsWith(operatingSystem, 'macOS') or startsWith(operatingSystem, 'iOS') or startsWith(operatingSystem, 'iPadOS')",
                },
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting Activation Lock data from {self.endpoint + self.CONFIG_ENDPOINT}: {e}",
            )
            return None

        device_ids = [device["id"] for device in self.graph_data["value"]]
        device_data = self.batch_request(
            device_ids,
            "deviceManagement/managedDevices/",
            "?$select=id,deviceName,serialNumber,activationLockBypassCode",
        )
        devices = [
            {k: v for k, v in d.items() if k != "@odata.context"}
            for d in device_data
            if d["activationLockBypassCode"] is not None
        ]

        try:
            self.process_data(
                data=devices,
                filetype=self.filetype,
                path=self.path,
                name_key=None,
                log_message=None,
            )
        except Exception as e:
            self.log(tag="error", msg=f"Error processing Activation Lock data: {e}")
            return None



# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class DeviceConfigurationsUpdateModule(BaseUpdateModule):
    """A class used to update Intune Device Configurations

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/deviceConfigurations/"

    def __init__(self, *args, **kwargs):
        """Initializes the DeviceConfigurationsUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Device Configurations/"
        self.assignment_endpoint = "/deviceManagement/deviceConfigurations/"
        self.assignment_extra_url = "/assign"
        self.exclude_paths = [
            "root['assignments']",
            "root['payload']",
            "root['isEncrypted']",
            "root['omaSettings']",
        ]

    def _handle_custom_apple_device_configuration(
        self, intune_data: dict[str, any], repo_data: dict[str, any]
    ) -> None:
        payload_path = self.path + "/mobileconfig/" + repo_data["payloadFileName"]
        if self.path_exists(payload_path):
            repo_payload_data = self.load_plist(payload_path)
            intune_decoded_payload_data = self.decode_base64(intune_data["payload"])
            self.write_temp_file(intune_decoded_payload_data)
            intune_payload_data = self.load_plist(self.path + "temp.mobileconfig")

            payload_diff = self.get_diffs(repo_payload_data, intune_payload_data, None)
            if payload_diff:
                self.update_downstream_data(
                    self.endpoint + self.CONFIG_ENDPOINT + self.downstream_id,
                    "patch",
                    204,
                    repo_data,
                )

                self.update_diff_data(payload_diff)

            self.remove_temp_file(self.path + "temp.mobileconfig")

    def _handle_custom_windows_device_configuration(
        self, intune_data: dict[str, any], repo_data: dict[str, any]
    ) -> None:
        omas = []
        oma_diff = None
        for setting in intune_data.get("omaSettings"):
            if setting["isEncrypted"]:
                decoded_oma = {}
                oma_value = self.make_graph_request(
                    endpoint=self.endpoint
                    + self.CONFIG_ENDPOINT
                    + "/"
                    + self.downstream_id
                    + "/getOmaSettingPlainTextValue(secretReferenceValueId='"
                    + setting["secretReferenceValueId"]
                    + "')",
                )
                decoded_oma["@odata.type"] = setting["@odata.type"]
                decoded_oma["displayName"] = setting["displayName"]
                decoded_oma["description"] = setting["description"]
                decoded_oma["omaUri"] = setting["omaUri"]
                decoded_oma["value"] = oma_value["value"]
                decoded_oma["isEncrypted"] = True
                decoded_oma["secretReferenceValueId"] = None
                omas.append(decoded_oma)
            elif not setting["isEncrypted"]:
                omas.append(setting)

        intune_data.pop("omaSettings")
        intune_data["omaSettings"] = omas

        for intune_omaSetting, repo_omaSetting in zip(
            intune_data.get("omaSettings"),
            repo_data["omaSettings"],
        ):
            oma_diff = self.get_diffs(
                repo_omaSetting, intune_omaSetting, ["root['isEncrypted']"]
            )

        if oma_diff:
            self.update_downstream_data(
                self.endpoint + self.CONFIG_ENDPOINT + self.downstream_id,
                "patch",
                204,
                repo_data,
            )

            self.update_diff_data(oma_diff)

    def _prepare_repo_omas(self, repo_data: dict[str, any]) -> dict[str, any]:
        repo_omas = []
        for repo_omaSetting in repo_data["omaSettings"]:
            if isinstance(repo_omaSetting["value"], dict):
                repo_omaSetting = self.remove_keys(repo_omaSetting)
                repo_omaSetting["value"] = repo_omaSetting["value"]["value"]
                repo_omas.append(repo_omaSetting)
            else:
                repo_omaSetting = self.remove_keys(repo_omaSetting)
                repo_omas.append(repo_omaSetting)
        repo_data.pop("omaSettings")
        repo_data["omaSettings"] = repo_omas

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            try:
                intune_data = self.get_downstream_data(self.CONFIG_ENDPOINT)
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            self.downstream_assignments = self.batch_assignment(
                intune_data["value"],
                self.assignment_endpoint,
                "/assignments",
            )

            custom_apple_odata_types = [
                "#microsoft.graph.iosCustomConfiguration",
                "#microsoft.graph.macOSCustomConfiguration",
            ]

            for filename in os.listdir(self.path):
                self.notify = True
                self.config_type = "Device Configuration"

                repo_data = self.load_repo_data(filename)
                if repo_data:
                    if "@odata.type" not in repo_data:
                        continue
                    self.match_info = {
                        "displayName": repo_data.get("displayName"),
                        "@odata.type": repo_data.get("@odata.type"),
                    }
                    self.name = repo_data.get("displayName")
                    diff_data = self.create_diff_data(self.name, self.config_type)

                    if (
                        repo_data["@odata.type"]
                        == "#microsoft.graph.windows10CustomConfiguration"
                    ):
                        self._prepare_repo_omas(repo_data)

                    try:
                        self.process_update(
                            downstream_data=intune_data["value"],
                            repo_data=repo_data,
                            method="patch",
                            status_code=204,
                            config_endpoint=self.CONFIG_ENDPOINT,
                        )
                    except Exception as e:
                        self.log(
                            tag="error",
                            msg=f"Error updating {self.config_type} {self.name}: {e}",
                        )

                    if self.downstream_object:
                        if repo_data["@odata.type"] in custom_apple_odata_types:
                            self.notify = False
                            self.config_type = "Mobileconfig"
                            self._handle_custom_apple_device_configuration(
                                self.downstream_object, repo_data
                            )
                        if (
                            repo_data["@odata.type"]
                            == "#microsoft.graph.windows10CustomConfiguration"
                        ):
                            self.notify = False
                            self.config_type = "OMA-URI"
                            self._handle_custom_windows_device_configuration(
                                self.downstream_object, repo_data
                            )

                    self.set_diff_data(diff_data)
                    self.diff_summary.append(diff_data)
                    self.reset_diffs_and_count()

            self.remove_downstream_data(self.CONFIG_ENDPOINT, intune_data["value"])

        return self.diff_summary



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
This module tests the get_authparams function.
"""

import unittest
from unittest.mock import patch

from testfixtures import TempDirectory

from src.IntuneCD.intunecdlib.get_authparams import getAuth


@patch("src.IntuneCD.intunecdlib.get_authparams.getAuth")
@patch(
    "src.IntuneCD.intunecdlib.get_authparams.obtain_accesstoken_app",
    return_value="token",
)
@patch(
    "src.IntuneCD.intunecdlib.get_authparams.obtain_accesstoken_cert",
    return_value="token",
)
@patch(
    "src.IntuneCD.intunecdlib.get_authparams.obtain_accesstoken_interactive",
    return_value="token",
)
class TestGetAuth(unittest.TestCase):
    """Test class for get_auth."""

    def setUp(self):
        self.directory = TempDirectory()
        self.directory.create()
        self.directory.write(
            "auth_dev.json",
            '{"params": {"DEV_CLIENT_ID": "test", "DEV_CLIENT_SECRET": "test", "DEV_TENANT_NAME": "test"}}',
            encoding="utf-8",
        )
        self.directory.write(
            "auth_prod.json",
            '{"params": {"PROD_CLIENT_ID": "test", "PROD_CLIENT_SECRET": "test", "PROD_TENANT_NAME": "test"}}',
            encoding="utf-8",
        )
        self.directory.write(
            "auth.json",
            '{"params": {"CLIENT_ID": "test", "CLIENT_SECRET": "test", "TENANT_NAME": "test"}}',
            encoding="utf-8",
        )
        self.auth_dev_json = self.directory.path + "/auth_dev.json"
        self.auth_prod_json = self.directory.path + "/auth_prod.json"
        self.auth_json = self.directory.path + "/auth.json"

    def tearDown(self):
        self.directory.cleanup()

    def test_get_auth_devtoprod_env_dev_app(self, _, __, ___, ____):
        """The auth params should be returned."""
        with patch.dict(
            "os.environ",
            {
                "DEV_CLIENT_ID": "test",
                "DEV_CLIENT_SECRET": "test",
                "DEV_TENANT_NAME": "test",
            },
        ):
            result = getAuth(
                "devtoprod",
                localauth=None,
                certauth=None,
                interactiveauth=None,
                scopes=[],
                entra=False,
                tenant="DEV",
            )
            self.assertEqual(result, "token")

    def test_get_auth_devtoprod_env_prod(self, _, __, ___, ____):
        """The auth params should be returned."""
        with patch.dict(
            "os.environ",
            {
                "PROD_CLIENT_ID": "test",
                "PROD_CLIENT_SECRET": "test",
                "PROD_TENANT_NAME": "test",
            },
        ):
            result = getAuth(
                "devtoprod",
                localauth=None,
                certauth=None,
                interactiveauth=None,
                scopes=[],
                entra=False,
                tenant="PROD",
            )
            self.assertEqual(result, "token")

    def test_get_auth_devtoprod_localauth_dev(self, _, __, ___, ____):
        """The auth params should be returned."""
        result = getAuth(
            "devtoprod",
            localauth=self.auth_dev_json,
            certauth=None,
            interactiveauth=None,
            scopes=[],
            entra=False,
            tenant="DEV",
        )
        self.assertEqual(result, "token")

    def test_get_auth_devtoprod_localauth_prod(self, _, __, ___, ____):
        """The auth params should be returned."""
        result = getAuth(
            "devtoprod",
            localauth=self.auth_prod_json,
            certauth=None,
            interactiveauth=None,
            scopes=[],
            entra=False,
            tenant="PROD",
        )
        self.assertEqual(result, "token")

    def test_get_auth_standalone_env(self, _, __, ___, ____):
        """The auth params should be returned."""
        with patch.dict(
            "os.environ",
            {"CLIENT_ID": "test", "CLIENT_SECRET": "test", "TENANT_NAME": "test"},
        ):
            result = getAuth(
                "standalone",
                localauth=None,
                certauth=None,
                interactiveauth=None,
                scopes=[],
                entra=False,
                tenant=None,
            )
            self.assertEqual(result, "token")

    def test_get_auth_standalone_localauth(self, _, __, ___, ____):
        """The auth params should be returned."""
        result = getAuth(
            "standalone",
            localauth=self.auth_json,
            certauth=None,
            interactiveauth=None,
            scopes=[],
            entra=False,
            tenant=None,
        )
        self.assertEqual(result, "token")

    def test_get_auth_devtoprod_missing_env_dev(self, _, __, ___, ____):
        """Exception should be raised due to missing env."""
        with patch.dict(
            "os.environ", {"DEV_CLIENT_ID": "test", "DEV_CLIENT_SECRET": "test"}
        ):
            with self.assertRaises(Exception):
                getAuth(
                    "devtoprod",
                    localauth=None,
                    certauth=None,
                    interactiveauth=None,
                    scopes=[],
                    entra=False,
                    tenant="DEV",
                )

    def test_get_auth_devtoprod_missing_env_prod(self, _, __, ___, ____):
        """Exception should be raised due to missing env."""
        with patch.dict(
            "os.environ", {"PROD_CLIENT_ID": "test", "PROD_CLIENT_SECRET": "test"}
        ):
            with self.assertRaises(Exception):
                getAuth(
                    "devtoprod",
                    localauth=None,
                    certauth=None,
                    interactiveauth=None,
                    scopes=[],
                    entra=False,
                    tenant="PROD",
                )

    def test_get_auth_standalone_missing_env(self, _, __, ___, ____):
        """Exception should be raised due to missing env."""
        with patch.dict("os.environ", {"CLIENT_ID": "test", "CLIENT_SECRET": "test"}):
            with self.assertRaises(Exception):
                getAuth(
                    "standalone",
                    localauth=None,
                    certauth=None,
                    interactiveauth=None,
                    scopes=[],
                    entra=False,
                    tenant=None,
                )

    def test_get_auth_cert(self, _, __, ___, ____):
        """The auth params should be returned."""
        with patch.dict(
            "os.environ",
            {
                "CLIENT_ID": "test",
                "TENANT_NAME": "test",
                "KEY_FILE": "test",
                "THUMBPRINT": "test",
            },
        ):
            result = getAuth(
                None,
                localauth=None,
                certauth=True,
                interactiveauth=None,
                scopes=[],
                tenant=None,
                entra=False,
            )
            self.assertEqual(result, "token")

    def test_get_auth_cert_missing_env(self, _, __, ___, ____):
        """The auth params should be returned."""
        with patch.dict(
            "os.environ",
            {"CLIENT_ID": "test", "TENANT_NAME": "test", "KEY_FILE": "test"},
        ):
            with self.assertRaises(Exception):
                getAuth(
                    None,
                    localauth=None,
                    certauth=True,
                    interactiveauth=None,
                    scopes=[],
                    entra=False,
                    tenant=None,
                )

    def test_get_auth_interactive(self, _, __, ___, ____):
        """The auth params should be returned."""
        with patch.dict(
            "os.environ",
            {
                "CLIENT_ID": "test",
                "TENANT_NAME": "test",
                "KEY_FILE": "test",
                "THUMBPRINT": "test",
            },
        ):
            result = getAuth(
                None,
                localauth=None,
                certauth=None,
                interactiveauth=True,
                scopes=[],
                tenant=None,
                entra=False,
            )
            self.assertEqual(result, "token")

    def test_get_auth_interactive_missing_env(self, _, __, ___, ____):
        """The auth params should be returned."""
        with patch.dict(
            "os.environ",
            {"CLIENT_ID": "test"},
        ):
            with self.assertRaises(Exception):
                getAuth(
                    None,
                    localauth=None,
                    certauth=None,
                    interactiveauth=True,
                    scopes=[],
                    entra=False,
                    tenant=None,
                )


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from .BaseGraphModule import BaseGraphModule


class ProcessScopeTags(BaseGraphModule):
    """Process scope tags from Intune."""

    def __init__(
        self,
        token: str = None,
    ):
        """Initializes the ProcessScopeTags class"""
        self.token = token

    def get_scope_tags(self):
        """
        Get scope tags from Intune.

        :param token: Token to use for authenticating the request
        """
        endpoint = "https://graph.microsoft.com/beta/deviceManagement/roleScopeTags"
        data = self.make_graph_request(endpoint)
        return data["value"]

    def get_scope_tags_name(self, data, scope_tags):
        """
        Get the name of the scope tag.

        :param data: The data to search for the scope tag name
        :param scope_tags: The scope tag to search for
        """

        def _get_scope_tags(self, scope_tag_key):
            self.log(
                function="get_scope_tags_name",
                msg="Checking if scope tags are in the data.",
            )
            # list comprehension to get the scope tag name
            if data.get(scope_tag_key):
                self.log(
                    function="get_scope_tags_name", msg="Scope tags are in the data."
                )
                data[scope_tag_key] = [
                    tag["displayName"]
                    for tag in scope_tags
                    if tag["id"] in data[scope_tag_key]
                ]
                self.log(
                    function="get_scope_tags_name",
                    msg=f"Scope tags: {data[scope_tag_key]}",
                )

        _get_scope_tags(self, "roleScopeTagIds")
        _get_scope_tags(self, "roleScopeTags")

        return data

    def get_scope_tags_id(self, data, scope_tags):
        """
        Get the ID of the scope tag.

        :param data: The data to search for the scope tag ID
        :param scope_tags: The scope tag to search for
        """

        def _get_scope_tags(self, scope_tag_key):
            self.log(
                function="get_scope_tags_id",
                msg="Checking if scope tags are in the data.",
            )
            # list comprehension to get the scope tag name
            if data.get(scope_tag_key):
                self.log(
                    function="get_scope_tags_id", msg="Scope tags are in the data."
                )
                data[scope_tag_key] = [
                    tag["id"]
                    for tag in scope_tags
                    if tag["displayName"] in data[scope_tag_key]
                ]
                self.log(
                    function="get_scope_tags_id",
                    msg=f"Scope tags: {data[scope_tag_key]}",
                )

        _get_scope_tags(self, "roleScopeTagIds")
        _get_scope_tags(self, "roleScopeTags")

        return data



# -*- coding: utf-8 -*-
import subprocess
import os

from .IntuneCDBase import IntuneCDBase


class ProcessAuditData(IntuneCDBase):
    """A class used to process the audit data from Intune."""

    def _git_installed(self):
        """
        Checks if git is installed.
        """
        cmd = ["git", "--version"]
        self.log(
            function="_git_installed",
            msg="Running command git --version to check if git is installed.",
        )
        git_version = subprocess.run(cmd, capture_output=True, text=True, check=False)
        if git_version.returncode != 0:
            self.log(function="_git_installed", msg="Git is not installed.")
            return False

        self.log(function="_git_installed", msg="Git is installed.")
        return True

    def _configure_git(self, audit_record, path):
        """
        Configures git with the user email and name.

        :param audit_record: The audit record to use for the configuration.
        :param path: The path to the git repo.
        """
        cmd = [
            "git",
            "-C",
            path,
            "config",
            "--local",
            "user.email",
            f"{audit_record['actor']}",
        ]
        self.log(
            function="_configure_git",
            msg=f"Running command {cmd} to configure git user email to {audit_record['actor']}.",
        )
        subprocess.run(
            cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True, check=False
        )

        # configure the user name for git commits
        cmd = [
            "git",
            "-C",
            path,
            "config",
            "--local",
            "user.name",
            f"{audit_record['actor']}",
        ]
        self.log(
            function="_configure_git",
            msg=f"Running command {cmd} to configure git user name.",
        )
        subprocess.run(
            cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True, check=False
        )

    def _check_if_git_repo(self, path, file):
        """
        Checks if the path is a git repo.

        :param path: The path to check.
        :param file: The file to check.
        """
        cmd = ["git", "-C", path, "rev-parse", "--is-inside-work-tree"]
        self.log(
            function="_check_if_git_repo",
            msg=f"Path is set to {path} and file is set to {file}.",
        )
        self.log(
            function="_check_if_git_repo",
            msg=f"Running command git command {cmd} to determine if the path is a git repo.",
        )
        git_status = subprocess.run(cmd, capture_output=True, text=True, check=False)

        if git_status.stdout.strip() == "true":
            self.log(function="_check_if_git_repo", msg="Path is a git repo.")
            return True

        self.log(function="_check_if_git_repo", msg="Path is not a git repo.")
        return False

    def _git_check_modified(self, path, file):
        """
        Checks if the file has been modified.

        :param path: The path to the git repo.
        :param file: The file to check.
        """
        cmd = ["git", "-C", path, "diff", "--name-only", f"{file}"]
        self.log(
            function="_git_check_modified",
            msg=f"Running command {cmd} to check if {file} has been modified.",
        )
        result = subprocess.run(cmd, capture_output=True, text=True, check=False)
        return result.stdout

    def _git_check_new_file(self, path, file):
        """
        Checks if the file is a new file.

        :param path: The path to the git repo.
        :param file: The file to check.
        """
        # check if it is a new file
        cmd = ["git", "-C", path, "ls-files", "--error-unmatch", f"{file}"]
        self.log(
            function="_git_check_new_file",
            msg=f"Running command {cmd} to check if {file} is a new file.",
        )
        new_file_result = subprocess.run(
            cmd, capture_output=True, text=True, check=False
        )
        # check if "did not match any file(s) known to git" is in the stderr
        if "did not match any file(s) known to git" in new_file_result.stderr:
            return True

        return False

    def _git_check_deleted_file(self, path, file):
        """Checks if the file is deleted.

        Args:
            path: The path to the git repo.
            file: The file to check.
        """

        # check if it is a deleted file
        cmd = ["git", "-C", path, "ls-files", "--deleted", f"{file}"]
        self.log(
            function="_git_check_deleted_file",
            msg=f"Running command {cmd} to check if {file} is a deleted file.",
        )
        deleted_file_result = subprocess.run(
            cmd, capture_output=True, text=True, check=False
        )
        # check if "deleted" is in the stdout
        if deleted_file_result.stdout:
            return True

        return False

    def _git_commit_changes(self, audit_record, path, file, deleted=False):
        """
        Commits the changes to the git repo.

        :param audit_record: The audit record to use for the commit.
        :param path: The path to the git repo.
        :param file: The file to commit.
        """
        # commit the changes
        if deleted:
            cmd = ["git", "-C", path, "rm", f"{file}"]
        else:
            cmd = ["git", "-C", path, "add", f"{file}"]
        self.log(
            function="_git_commit_changes",
            msg=f"Running command {cmd} to add {file} to the git repo.",
        )
        subprocess.run(
            cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True, check=False
        )
        self.log(
            function="_git_commit_changes", msg=f"Committing the changes to {file}."
        )
        cmd = [
            "git",
            "-C",
            path,
            "commit",
            "-m",
            (
                f"{audit_record['activityType']} by {audit_record['actor']}\n"
                f"Date: {audit_record['activityDateTime']}\n"
                f"result: {audit_record['activityResult']}"
            ),
        ]

        commit = subprocess.run(
            cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True, check=False
        )

        if commit.returncode == 0:
            self.log(function="_git_commit_changes", msg="Commit was successful.")
        else:
            self.log(
                function="_git_commit_changes",
                tag="error",
                msg=f"Commit was not successful, error: {commit.stderr[:200] or commit.stdout[:200]}",
            )

    def _get_payload_from_audit_data(self, audit_data, compare_data):
        """
        Gets the payload from the audit data.

        :param audit_data: The audit data to get the payload from.
        :param compare_data: The data to compare the audit data to.
        """

        records = []
        for record in audit_data:
            if record[compare_data["type"]] == compare_data["value"]:
                records.append(record)

        if records:
            # sort the records by activityDateTime
            records.sort(key=lambda x: x["activityDateTime"], reverse=True)
            records = records[0]

        return records

    def process_audit_data(
        self,
        audit_data,
        compare_data,
        path,
        file,
        get_record=True,
        record=None,
        source_file=None,
    ):
        """
        Processes the audit data from Intune.

        :param audit_data: The audit data to process.
        :param compare_data: The data to compare the audit data to.
        :param path: The path to the file.
        :param file: The file to process.
        """

        self.log(
            function="process_audit_data",
            msg=f"Processing audit data for {file} in path {path}.",
        )
        # determine if the path we are using is a git repo
        git_repo = self._check_if_git_repo(path, file)

        # Commit the changes
        if git_repo:
            if get_record:
                record = self._get_payload_from_audit_data(audit_data, compare_data)
            if not record:
                self.log(
                    function="process_audit_data",
                    msg=f"No audit data found for {file}.",
                )
                return False
            # check if git is installed
            if not self._git_installed():
                return
            # configure git
            self._configure_git(record, path)
            # check if file has been modified
            result = self._git_check_modified(path, file)

            if not result:
                file_not_found = self._git_check_new_file(path, file)
                if source_file:
                    path = os.path.dirname(source_file)
                    file_deleted = self._git_check_deleted_file(path, source_file)
                else:
                    file_deleted = self._git_check_deleted_file(path, file)

            if result or file_not_found or file_deleted:
                # commit the changes
                if file_deleted and source_file:
                    path = os.path.dirname(source_file)
                    self._git_commit_changes(record, path, source_file, deleted=True)
                elif file_deleted:
                    self._git_commit_changes(record, path, file, deleted=True)
                else:
                    self._git_commit_changes(record, path, file)
            else:
                self.log(
                    function="process_audit_data",
                    msg=f"{file} has not been modified, no changes to commit.",
                )

        self.log(function="process_audit_data", msg="Audit data has been processed.")



# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseBackupModule import BaseBackupModule


class ProactiveRemediationScriptBackupModule(BaseBackupModule):
    """A class used to backup Intune Proactive Remediation Scripts

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        LOG_MESSAGE (str): The message to log when backing up the data
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/deviceHealthScripts"
    LOG_MESSAGE = "Backing up Proactive Remediation: "

    def __init__(self, *args, **kwargs):
        """Initializes the ProactiveRemediationScriptBackupModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Proactive Remediations/"
        self.script_data_path = f"{self.path}/Script Data/"
        self.audit_filter = (
            self.audit_filter or "componentName eq 'DeviceConfiguration'"
        )
        self.assignment_endpoint = "deviceManagement/deviceHealthScripts/"
        self.assignment_extra_url = "/assignments"

    def _save_script(self, item: dict, script_type: str) -> None:
        """Saves the script to a file

        Args:
            item (dict): The script data
            script_type (str): The type of script to save
        """
        if self.prefix:
            match = self.check_prefix_match(item["displayName"], self.prefix)
            if not match:
                return
        try:
            if not os.path.exists(self.script_data_path):
                os.makedirs(self.script_data_path)
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error creating directory {self.script_data_path}: {e}",
            )

        # Save detection script to the Script Data folder
        fname = self._prepare_file_name(item["displayName"])
        if self.append_id:
            fname_id = f"__{item['id']}"
        else:
            fname_id = ""
        decoded = self.decode_base64(item[f"{script_type}Content"])
        try:
            with open(
                f"{self.script_data_path}{fname}_{script_type}{fname_id}.ps1",
                "w",
                encoding="utf-8",
            ) as f:
                f.write(decoded)
        except Exception as e:
            self.log(tag="error", msg=f"Error writing script {fname} to file: {e}")

    def main(self) -> dict[str, any]:
        """The main method to backup the Proactive Remediation Scripts

        Returns:
            dict[str, any]: The results of the backup
        """
        try:
            self.graph_data = self.make_graph_request(
                endpoint=self.endpoint + self.CONFIG_ENDPOINT
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting Proactive Remediation data from {self.endpoint + self.CONFIG_ENDPOINT}: {e}",
            )
            return None

        proactiveremediation_ids = [item["id"] for item in self.graph_data["value"]]
        # Get the script data details using batch request
        proactiveremediations_responses = self.batch_request(
            proactiveremediation_ids, "deviceManagement/deviceHealthScripts/", ""
        )
        # Get items that are not from Microsoft
        proactiveremediations_responses = [
            item
            for item in proactiveremediations_responses
            if "Microsoft" not in item["publisher"]
        ]

        # Save the script data to file
        for item in proactiveremediations_responses:
            self._save_script(item, "detectionScript")
            self._save_script(item, "remediationScript")

        try:
            self.results = self.process_data(
                data=proactiveremediations_responses,
                filetype=self.filetype,
                path=self.path,
                name_key="displayName",
                log_message=self.LOG_MESSAGE,
                audit_compare_info={"type": "resourceId", "value_key": "id"},
            )
        except Exception as e:
            self.log(
                tag="error", msg=f"Error processing Proactive Remediation data: {e}"
            )
            return None

        return self.results



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
This module moves files to archive during backup if they have been removed from Intune.
"""

import os
import shutil

from datetime import datetime, timedelta, timezone
from .BaseBackupModule import BaseBackupModule
from .process_audit_data import ProcessAuditData


class Archive(BaseBackupModule):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.exclude = {
            "Management Intents",
            "archive",
            "__archive__",
            "Assignment Report",
            "Autopilot Devices",
            "Activation Lock Bypass Codes",
        }
        self.date_tag = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
        self.process_audit_data = ProcessAuditData()
        self.audit_endpoint = (
            "https://graph.microsoft.com/beta/deviceManagement/auditEvents"
        )
        if self.append_id and self.audit:
            self.audit_data = self._get_audit_delete_events()

    def archive_file(self, file, root):
        archive_path = os.path.join(self.path, "__archive__", self.date_tag)
        if not os.path.exists(archive_path):
            os.makedirs(archive_path)

        src = os.path.join(root, file)
        dst = os.path.join(archive_path, file)
        shutil.move(src, dst)

        if self.audit_data:
            self._handle_audit_commit(file, dst, archive_path, src)

    def _get_audit_delete_events(self) -> list:
        """Gets all delete events from the audit log from the last 24h

        Returns:
            list: A list of all delete events
        """
        if not os.getenv("AUDIT_DAYS_BACK"):
            days_back = 1
        else:
            days_back = int(os.getenv("AUDIT_DAYS_BACK"))
        start_date = (
            datetime.now(timezone.utc) - timedelta(days=days_back)
        ).isoformat()
        end_date = datetime.now(timezone.utc).isoformat()

        q_params = {
            "$filter": (
                f"activityOperationType eq 'Delete' and "
                f"activityDateTime gt {start_date} and "
                f"activityDateTime le {end_date}"
            ),
            "$select": "actor,activityDateTime,activityType,activityOperationType,activityResult,resources",
            "$orderby": "activityDateTime desc",
        }

        audit_data = self.make_graph_request(
            self.audit_endpoint, params=q_params, method="GET"
        )

        return audit_data

    def _handle_audit_commit(self, filename, filepath, archive_path, source_file):
        """Handles the audit commit for the file

        Args:
            filename: The name of the file
            filepath: The path to the file
            archive_path: The path to the archive
        """
        resource_id = filename.split("__")[-1].replace(".json", "").replace(".yaml", "")

        if self.audit_data:
            audit_data_record = next(
                (
                    item
                    for item in self.audit_data.get("value", [])
                    if resource_id
                    in [res.get("resourceId") for res in item.get("resources", [])]
                ),
                None,
            )

        if audit_data_record:
            if audit_data_record["actor"]["auditActorType"] == "ItPro":
                actor = audit_data_record["actor"].get("userPrincipalName")
            else:
                actor = audit_data_record["actor"].get("applicationDisplayName")

            audit_data_record = {
                "resourceId": audit_data_record["resources"][0]["resourceId"],
                "auditResourceType": audit_data_record["resources"][0][
                    "auditResourceType"
                ],
                "actor": actor,
                "activityDateTime": audit_data_record["activityDateTime"],
                "activityType": audit_data_record["activityType"],
                "activityOperationType": audit_data_record["activityOperationType"],
                "activityResult": audit_data_record["activityResult"],
            }

            self.filename = os.path.splitext(os.path.basename(filepath))[0]
            # process audit and check for deleted files
            self.process_audit_data.process_audit_data(
                audit_data_record,
                None,
                archive_path,
                filepath,
                get_record=False,
                record=audit_data_record,
                source_file=source_file,
            )
            # process audit and check for archived files
            self.process_audit_data.process_audit_data(
                audit_data_record,
                None,
                archive_path,
                filepath,
                get_record=False,
                record=audit_data_record,
            )

    def move_to_archive(self, created_files):
        if not os.path.exists(os.path.join(self.path, "__archive__")):
            os.makedirs(os.path.join(self.path, "__archive__"))

        for root, dirs, files in os.walk(self.path, topdown=True):
            dirs[:] = [d for d in dirs if d not in self.exclude]
            for file in files:
                if file.endswith((".yaml", ".json")):
                    if root == self.path and file.endswith(".json"):
                        continue
                    if file.replace(f".{self.filetype}", "") not in created_files:
                        self.archive_file(file, root)

        mgmt_path = os.path.join(self.path, "Management Intents")
        if os.path.exists(mgmt_path):
            for root, dirs, files in os.walk(mgmt_path, topdown=True):
                for file in files:
                    if file.endswith((".yaml", ".json")):
                        if file.replace(f".{self.filetype}", "") not in created_files:
                            self.archive_file(file, root)
